@@ -1,3 +1,59 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2005 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*
  * smsc.c - implement interface to SMS centers as defined by smsc.h
  *
@@ -73,35 +129,13 @@ SMSCenter *smscenter_construct(void)
     smsc->cimd_username = NULL;
     smsc->cimd_password = NULL;
 
-    /* CIMD 2 */
-    smsc->cimd2_hostname = NULL;
-    smsc->cimd2_port = -1;
-    smsc->cimd2_username = NULL;
-    smsc->cimd2_password = NULL;
-    smsc->cimd2_send_seq = 1;
-    smsc->cimd2_receive_seq = 0;
-    smsc->cimd2_inbuffer = NULL;
-    smsc->cimd2_received = NULL;
-    smsc->cimd2_error = 0;
-    smsc->cimd2_next_ping = 0;
-
-    /* EMI */
+    /* EMI_X25 */
     smsc->emi_phonenum = NULL;
     smsc->emi_serialdevice = NULL;
     smsc->emi_username = NULL;
     smsc->emi_password = NULL;
 
-    /* EMI IP */
-    smsc->emi_hostname = NULL;
-    smsc->emi_port = -1;
-    smsc->emi_backup_fd = -1;
-    smsc->emi_backup_port = -1;
-    smsc->emi_backup_allow_ip = NULL;
-    smsc->emi_our_port = -1;
-    smsc->emi_secondary_fd = -1;
-
     /* SEMA SMS2000 */
-
     smsc->sema_smscnua = NULL;
     smsc->sema_homenua = NULL;
     smsc->sema_serialdevice = NULL;
@@ -119,17 +153,6 @@ SMSCenter *smscenter_construct(void)
     smsc->ois_bufsize = 0;
     smsc->ois_buffer = 0;
 
-    /* AT Wireless modems  (GSM 03.40 version 7.4.0) */
-
-    smsc->at_serialdevice = NULL;
-    smsc->at_fd = -1;
-    smsc->at_modemtype = NULL;
-    smsc->at_received = NULL;
-    smsc->at_inbuffer = NULL;
-    smsc->at_pin = NULL;
-    smsc->at_validityperiod = NULL;
-    smsc->at_alt_dcs = 0;
-
     /* add new SMSCes here */
 
     /* Memory */
@@ -155,24 +178,12 @@ void smscenter_destruct(SMSCenter *smsc)
     gw_free(smsc->cimd_username);
     gw_free(smsc->cimd_password);
 
-    /* CIMD 2 */
-    octstr_destroy(smsc->cimd2_hostname);
-    octstr_destroy(smsc->cimd2_username);
-    octstr_destroy(smsc->cimd2_password);
-    octstr_destroy(smsc->cimd2_inbuffer);
-    octstr_destroy(smsc->sender_prefix);
-    list_destroy(smsc->cimd2_received, NULL);
-
-    /* EMI */
+    /* EMI_X25 */
     gw_free(smsc->emi_phonenum);
     gw_free(smsc->emi_serialdevice);
     gw_free(smsc->emi_username);
     gw_free(smsc->emi_password);
 
-    /* EMI IP */
-    gw_free(smsc->emi_hostname);
-    gw_free(smsc->emi_backup_allow_ip);
-
     /* SEMA */
     gw_free(smsc->sema_smscnua);
     gw_free(smsc->sema_homenua);
@@ -182,13 +193,6 @@ void smscenter_destruct(SMSCenter *smsc)
     ois_delete_queue(smsc);
     gw_free(smsc->ois_buffer);
 
-    /* AT */
-    gw_free(smsc->at_serialdevice);
-    gw_free(smsc->at_modemtype);
-    gw_free(smsc->at_pin);
-    list_destroy(smsc->at_received, NULL);
-    gw_free(smsc->at_inbuffer);
-
     /* add new SMSCes here */
 
     /* Other fields */
@@ -213,13 +217,7 @@ int smscenter_submit_msg(SMSCenter *smsc, Msg *msg)
             goto error;
         break;
 
-    case SMSC_TYPE_CIMD2:
-        if (cimd2_submit_msg(smsc, msg) == -1)
-            goto error;
-        break;
-
-    case SMSC_TYPE_EMI:
-    case SMSC_TYPE_EMI_IP:
+    case SMSC_TYPE_EMI_X25:
         if (emi_submit_msg(smsc, msg) == -1)
             goto error;
         break;
@@ -234,11 +232,6 @@ int smscenter_submit_msg(SMSCenter *smsc, Msg *msg)
             goto error;
         break;
 
-    case SMSC_TYPE_AT:
-        if (at_submit_msg(smsc, msg) == -1)
-            goto error;
-        break;
-
         /* add new SMSCes here */
 
     default:
@@ -269,14 +262,7 @@ int smscenter_receive_msg(SMSCenter *smsc, Msg **msg)
             goto error;
         break;
 
-    case SMSC_TYPE_CIMD2:
-        ret = cimd2_receive_msg(smsc, msg);
-        if (ret == -1)
-            goto error;
-        break;
-
-    case SMSC_TYPE_EMI:
-    case SMSC_TYPE_EMI_IP:
+    case SMSC_TYPE_EMI_X25:
         ret = emi_receive_msg(smsc, msg);
         if (ret == -1)
             goto error;
@@ -295,12 +281,6 @@ int smscenter_receive_msg(SMSCenter *smsc, Msg **msg)
             goto error;
         break;
 
-    case SMSC_TYPE_AT:
-        ret = at_receive_msg(smsc, msg);
-        if (ret == -1)
-            goto error;
-        break;
-
     default:
         goto error;
 
@@ -333,14 +313,7 @@ int smscenter_pending_smsmessage(SMSCenter *smsc)
             goto error;
         break;
 
-    case SMSC_TYPE_CIMD2:
-        ret = cimd2_pending_smsmessage(smsc);
-        if (ret == -1)
-            goto error;
-        break;
-
-    case SMSC_TYPE_EMI:
-    case SMSC_TYPE_EMI_IP:
+    case SMSC_TYPE_EMI_X25:
         ret = emi_pending_smsmessage(smsc);
         if (ret == -1)
             goto error;
@@ -358,12 +331,6 @@ int smscenter_pending_smsmessage(SMSCenter *smsc)
             goto error;
         break;
 
-    case SMSC_TYPE_AT:
-        ret = at_pending_smsmessage(smsc);
-        if (ret == -1)
-            goto error;
-        break;
-
     default:
         goto error;
     }
@@ -482,7 +449,6 @@ SMSCenter *smsc_open(CfgGroup *grp)
     Octstr *preferred_prefix, *allowed_prefix, *denied_prefix;
     Octstr *alt_chars, *allow_ip;
     Octstr *sema_smscnua, *sema_homenua, *sema_report;
-    Octstr *at_modemtype, *at_pin, *at_validityperiod;
     Octstr *sender_prefix;
 
     long iwaitreport;
@@ -500,18 +466,12 @@ SMSCenter *smsc_open(CfgGroup *grp)
     }
     if (octstr_compare(type, octstr_imm("cimd")) == 0)
     	typeno = SMSC_TYPE_CIMD;
-    else if (octstr_compare(type, octstr_imm("cimd2")) == 0)
-    	typeno = SMSC_TYPE_CIMD2;
-    else if (octstr_compare(type, octstr_imm("emi")) == 0)
-    	typeno = SMSC_TYPE_EMI;
-    else if (octstr_compare(type, octstr_imm("emi_ip")) == 0)
-    	typeno = SMSC_TYPE_EMI_IP;
+    else if (octstr_compare(type, octstr_imm("emi_x25")) == 0)
+    	typeno = SMSC_TYPE_EMI_X25;
     else if (octstr_compare(type, octstr_imm("sema")) == 0)
     	typeno = SMSC_TYPE_SEMA_X28;
     else if (octstr_compare(type, octstr_imm("ois")) == 0)
     	typeno = SMSC_TYPE_OIS;
-    else if (octstr_compare(type, octstr_imm("at")) == 0)
-    	typeno = SMSC_TYPE_AT;
     else {
 	error(0, "Unknown SMSC type '%s'", octstr_get_cstr(type));
 	octstr_destroy(type);
@@ -555,10 +515,6 @@ SMSCenter *smsc_open(CfgGroup *grp)
     if (cfg_get_integer(&ois_debug, grp, octstr_imm("ois-debug-level")) == -1)
     	ois_debug = 0;
 
-    at_modemtype = cfg_get(grp, octstr_imm("modemtype"));
-    at_pin = cfg_get(grp, octstr_imm("pin"));
-    at_validityperiod = cfg_get(grp, octstr_imm("validityperiod"));
-
     sender_prefix = cfg_get(grp, octstr_imm("sender-prefix"));
     if (sender_prefix == NULL)
         sender_prefix = octstr_create("never");
@@ -576,22 +532,10 @@ SMSCenter *smsc_open(CfgGroup *grp)
 			     octstr_get_cstr(password));
         break;
 
-    case SMSC_TYPE_CIMD2:
-        if (host == NULL || port == 0 || username == NULL || password == NULL)
-            error(0, "Required field missing for CIMD 2 center.");
-        else
-            smsc = cimd2_open(host,
-	    	    	      port, 
-			      username, 
-			      password, 
-			      keepalive,
-                              sender_prefix);
-        break;
-
-    case SMSC_TYPE_EMI:
+    case SMSC_TYPE_EMI_X25:
         if (phone == NULL || device == NULL || username == NULL ||
             password == NULL)
-            error(0, "Required field missing for EMI center.");
+            error(0, "Required field missing for EMI_X25 center.");
         else
             smsc = emi_open(octstr_get_cstr(phone), 
 	    	    	    octstr_get_cstr(device), 
@@ -599,19 +543,6 @@ SMSCenter *smsc_open(CfgGroup *grp)
 			    octstr_get_cstr(password));
         break;
 
-    case SMSC_TYPE_EMI_IP:
-        if (host == NULL || port == 0)
-            error(0, "Required field missing for EMI IP center.");
-        else
-	    smsc = emi_open_ip(octstr_get_cstr(host), 
-	    	    	       port, 
-			       username ? octstr_get_cstr(username) : 0, 
-			       password ? octstr_get_cstr(password) : 0,
-                               receive_port, 
-			       allow_ip ? octstr_get_cstr(allow_ip) : 0, 
-			       our_port);
-        break;
-
     case SMSC_TYPE_SEMA_X28:
         if (device == NULL || sema_smscnua == NULL || sema_homenua == NULL)
             error(0, "Required field missing for SEMA center.");
@@ -632,18 +563,6 @@ SMSCenter *smsc_open(CfgGroup *grp)
 	    	    	    ois_debug);
         break;
 
-    case SMSC_TYPE_AT:
-        if (device == NULL)
-            error(0, "Required field missing for AT virtual center.");
-        else
-            smsc = at_open(octstr_get_cstr(device), 
-	    	    	   at_modemtype ? octstr_get_cstr(at_modemtype) : 0, 
-			   at_pin ? octstr_get_cstr(at_pin) : 0,
-	    	    	   at_validityperiod ? 
-			       octstr_get_cstr(at_validityperiod) : 0,
-			       alt_dcs);
-        break;
-
         /* add new SMSCes here */
 
     default: 		/* Unknown SMSC type */
@@ -683,9 +602,6 @@ SMSCenter *smsc_open(CfgGroup *grp)
     octstr_destroy(sema_smscnua);
     octstr_destroy(sema_homenua);
     octstr_destroy(sema_report);
-    octstr_destroy(at_modemtype);
-    octstr_destroy(at_pin);
-    octstr_destroy(at_validityperiod);
     octstr_destroy(sender_prefix);
     return smsc;
 }
@@ -705,13 +621,7 @@ int smsc_reopen(SMSCenter *smsc)
     case SMSC_TYPE_CIMD:
         ret = cimd_reopen(smsc);
 	break;
-    case SMSC_TYPE_CIMD2:
-        ret = cimd2_reopen(smsc);
-	break;
-    case SMSC_TYPE_EMI_IP:
-        ret = emi_reopen_ip(smsc);
-	break;
-    case SMSC_TYPE_EMI:
+    case SMSC_TYPE_EMI_X25:
         ret = emi_reopen(smsc);
 	break;
     case SMSC_TYPE_SEMA_X28:
@@ -720,9 +630,6 @@ int smsc_reopen(SMSCenter *smsc)
     case SMSC_TYPE_OIS:
         ret = ois_reopen(smsc);
 	break;
-    case SMSC_TYPE_AT:
-        ret = at_reopen(smsc);
-	break;
         /* add new SMSCes here */
     default: 		/* Unknown SMSC type */
         ret = -2; 		/* no use */
@@ -754,21 +661,11 @@ int smsc_close(SMSCenter *smsc)
             errors = 1;
         break;
 
-    case SMSC_TYPE_CIMD2:
-        if (cimd2_close(smsc) == -1)
-            errors = 1;
-        break;
-
-    case SMSC_TYPE_EMI:
+    case SMSC_TYPE_EMI_X25:
         if (emi_close(smsc) == -1)
             errors = 1;
         break;
 
-    case SMSC_TYPE_EMI_IP:
-        if (emi_close_ip(smsc) == -1)
-            errors = 1;
-        break;
-
     case SMSC_TYPE_SEMA_X28:
         if (sema_close(smsc) == -1)
             errors = 1;
@@ -779,11 +676,6 @@ int smsc_close(SMSCenter *smsc)
             errors = 1;
         break;
 
-    case SMSC_TYPE_AT:
-        if (at_close(smsc) == -1)
-            errors = 1;
-        break;
-
         /* add new SMSCes here */
 
     default: 		/* Unknown SMSC type */
