@@ -1,3 +1,59 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2005 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*
  * smsc_cgw.c - Implements an interface to the Sonera ContentGateway software
  *
@@ -128,7 +184,7 @@ static Connection *cgw_open_send_connection(SMSCConn *conn);
 static int cgw_send_loop(SMSCConn *conn, Connection *server);
 void cgw_check_acks(PrivData *privdata);
 int cgw_wait_command(PrivData *privdata, SMSCConn *conn, Connection *server, int timeout);
-static int cgw_open_listening_socket(PrivData *privdata);
+static int cgw_open_listening_socket(SMSCConn *conn, PrivData *privdata);
 static void cgw_listener(void *arg);
 static void cgw_receiver(SMSCConn *conn, Connection *server);
 static int cgw_handle_op(SMSCConn *conn, Connection *server, struct cgwop *cgwop);
@@ -211,7 +267,7 @@ static Octstr *cgwop_tostr(struct cgwop *cgwop)
     int len = cgwop->num_fields;
     Octstr *str;
 
-    if (cgw_ops[cgwop->op] == NULL) return NULL;     // invalid operation
+    if (cgw_ops[cgwop->op] == NULL) return NULL;     /* invalid operation */
 
     str = octstr_create("");
 
@@ -263,12 +319,12 @@ static Octstr *cgw_encode_msg(Octstr* str)
        length is under 160 chars (the checking could probably be done better) */
 
     while ((i = octstr_search_char(str, 0x80, 0)) != -1) {    
-        octstr_delete(str, i, 1);     // delete Euro char
+        octstr_delete(str, i, 1);     /* delete Euro char */
 	if (octstr_len(str) < 160) {
-	    octstr_insert_data(str, i, &esc, 1);  // replace with ESC + e
+	    octstr_insert_data(str, i, &esc, 1);  /* replace with ESC + e */
 	    octstr_insert_data(str, i+1, &e, 1);  
 	} else {
-	    octstr_insert_data(str, i, &e, 1);  // no room for ESC + e, just replace with an e
+	    octstr_insert_data(str, i, &e, 1);  /* no room for ESC + e, just replace with an e */
         }
     }
 
@@ -279,12 +335,12 @@ static Octstr *cgw_encode_msg(Octstr* str)
     }
     /* Remove Line Feed characters */
     while ((i = octstr_search_char(str, CGW_EOL, 0)) != -1) {
-        octstr_delete(str, i, 1);     // delete EOL char
+        octstr_delete(str, i, 1);     /* delete EOL char */
         octstr_insert(str, octstr_imm("\\n"), i);
     }
     /* Remove Carriage return characters */
     while ((i = octstr_search_char(str, 13, 0)) != -1) {
-        octstr_delete(str, i, 1);     // delete EOL char
+        octstr_delete(str, i, 1);     /* delete EOL char */
         octstr_insert(str, octstr_imm("\\r"), i);
     }
 
@@ -301,12 +357,12 @@ static Octstr *cgw_decode_msg(Octstr* str)
 
     /* make \n -> linefeed */
     while ((i = octstr_search(str, octstr_imm("\\n"), 0)) != -1) {
-        octstr_delete(str, i, 2);     // delete "\n" str
+        octstr_delete(str, i, 2);     /* delete "\n" str */
         octstr_insert(str, octstr_imm("\n"), i);
     }
     /* make \r -> carriage return */
     while ((i = octstr_search(str, octstr_imm("\\r"), 0)) != -1) {
-        octstr_delete(str, i, 2);     // delete EOL char
+        octstr_delete(str, i, 2);     /* delete EOL char */
         octstr_insert(str, octstr_imm("\r"), i);
     }
     /* remove double backslashes */
@@ -342,7 +398,7 @@ static struct cgwop *msg_to_cgwop(PrivData *privdata, Msg *msg, int trn)
     cgwop_add(cgwop, octstr_imm("to"), msg->sms.receiver);
 
     /* If delivery reports are asked, ask for them by adding a nrq:anything field */
-    if (msg->sms.dlr_mask & 0x07)
+    if (DLR_IS_ENABLED_DEVICE(msg->sms.dlr_mask))
         cgwop_add(cgwop, octstr_imm("nrq"), octstr_imm("true"));
 
     octstr_destroy(sender);
@@ -380,7 +436,7 @@ int smsc_cgw_create(SMSCConn *conn, CfgGroup *cfg)
     int i;
 
     privdata = gw_malloc(sizeof(PrivData));
-    privdata->outgoing_queue = list_create();
+    privdata->outgoing_queue = gwlist_create();
     privdata->listening_socket = -1;
 
     if (cfg_get_integer(&portno, cfg, octstr_imm("port")) == -1)
@@ -436,7 +492,7 @@ int smsc_cgw_create(SMSCConn *conn, CfgGroup *cfg)
         privdata->dlr[i] = 0;
     }
 
-    if (privdata->rport > 0 && cgw_open_listening_socket(privdata) < 0) {
+    if (privdata->rport > 0 && cgw_open_listening_socket(conn,privdata) < 0) {
         gw_free(privdata);
         privdata = NULL;
         goto error;
@@ -470,7 +526,7 @@ int smsc_cgw_create(SMSCConn *conn, CfgGroup *cfg)
 error:
     error(0, "Failed to create CGW smsc connection");
     if (privdata != NULL)
-        list_destroy(privdata->outgoing_queue, NULL);
+        gwlist_destroy(privdata->outgoing_queue, NULL);
 
     gw_free(privdata);
     octstr_destroy(host);
@@ -496,7 +552,7 @@ static int cgw_add_msg_cb(SMSCConn *conn, Msg *sms)
     Msg *copy;
 
     copy = msg_duplicate(sms);
-    list_produce(privdata->outgoing_queue, copy);
+    gwlist_produce(privdata->outgoing_queue, copy);
     gwthread_wakeup(privdata->sender_thread);
 
     return 0;
@@ -518,8 +574,8 @@ static int cgw_shutdown_cb(SMSCConn *conn, int finish_sending)
 
     if (finish_sending == 0) {
         Msg *msg;
-        while ((msg = list_extract_first(privdata->outgoing_queue)) != NULL) {
-            bb_smscconn_send_failed(conn, msg, SMSCCONN_FAILED_SHUTDOWN);
+        while ((msg = gwlist_extract_first(privdata->outgoing_queue)) != NULL) {
+            bb_smscconn_send_failed(conn, msg, SMSCCONN_FAILED_SHUTDOWN, NULL);
         }
     }
 
@@ -543,7 +599,7 @@ static void cgw_start_cb(SMSCConn *conn)
 static long cgw_queued_cb(SMSCConn *conn)
 {
     PrivData *privdata = conn->data;
-    long ret = list_len(privdata->outgoing_queue);
+    long ret = gwlist_len(privdata->outgoing_queue);
 
     /* use internal queue as load, maybe something else later */
 
@@ -568,9 +624,12 @@ static void cgw_sender(void *arg)
 
     conn->status = SMSCCONN_CONNECTING;
 
+    /* Make sure we log into our own log-file if defined */
+    log_thread_to(conn->log_idx);
+
     while (!privdata->shutdown) {
 
-        // check that connection is active
+        /* check that connection is active */
         if (conn->status != SMSCCONN_ACTIVE) {
             if ((server = cgw_open_send_connection(conn)) == NULL) {
                 privdata->shutdown = 1;
@@ -582,7 +641,7 @@ static void cgw_sender(void *arg)
             bb_smscconn_connected(conn);
         } else {
 	    ret = 0;
-            l = list_len(privdata->outgoing_queue);
+            l = gwlist_len(privdata->outgoing_queue);
             if (l > 0)
                ret = cgw_send_loop(conn, server);     /* send any messages in queue */
 
@@ -599,13 +658,13 @@ static void cgw_sender(void *arg)
 
     conn_destroy(server);
 
-    while ((msg = list_extract_first(privdata->outgoing_queue)) != NULL)
-        bb_smscconn_send_failed(conn, msg, SMSCCONN_FAILED_SHUTDOWN);
+    while ((msg = gwlist_extract_first(privdata->outgoing_queue)) != NULL)
+        bb_smscconn_send_failed(conn, msg, SMSCCONN_FAILED_SHUTDOWN, NULL);
     mutex_lock(conn->flow_mutex);
 
     conn->status = SMSCCONN_DEAD;
 
-    list_destroy(privdata->outgoing_queue, NULL);
+    gwlist_destroy(privdata->outgoing_queue, NULL);
     octstr_destroy(privdata->host);
     octstr_destroy(privdata->allow_ip);
     octstr_destroy(privdata->deny_ip);
@@ -639,8 +698,8 @@ static Connection *cgw_open_send_connection(SMSCConn *conn)
                 conn->status = SMSCCONN_RECONNECTING;
                 mutex_unlock(conn->flow_mutex);
             }
-            while ((msg = list_extract_first(privdata->outgoing_queue)))
-                bb_smscconn_send_failed(conn, msg, SMSCCONN_FAILED_TEMPORARILY);
+            while ((msg = gwlist_extract_first(privdata->outgoing_queue)))
+                bb_smscconn_send_failed(conn, msg, SMSCCONN_FAILED_TEMPORARILY, NULL);
             info(0, "smsc_cgw: waiting for %d minutes before trying to connect again", wait);
             gwthread_sleep(wait * 60);
             wait = wait > 5 ? 10 : wait * 2;
@@ -648,7 +707,7 @@ static Connection *cgw_open_send_connection(SMSCConn *conn)
             wait = 1;
 
         server = conn_open_tcp_with_port(privdata->host, privdata->port,
-                                         privdata->our_port, NULL /* privdata->our_host */);
+                                         privdata->our_port, conn->our_host);
 
         if (privdata->shutdown) {
             conn_destroy(server);
@@ -685,7 +744,7 @@ static int cgw_send_loop(SMSCConn *conn, Connection *server)
     int firsttrn;
 
     /* Send messages in queue */
-    while ((msg = list_extract_first(privdata->outgoing_queue)) != NULL) {
+    while ((msg = gwlist_extract_first(privdata->outgoing_queue)) != NULL) {
         firsttrn = privdata->nexttrn;
         while (privdata->sendtime[privdata->nexttrn] != 0) { 
             if (++privdata->nexttrn >= CGW_TRN_MAX) privdata->nexttrn = 0;    
@@ -694,7 +753,7 @@ static int cgw_send_loop(SMSCConn *conn, Connection *server)
 		 * haven't been acked. In this case, increase size of 
                  * CGW_TRN_MAX */
                 info(0, "cgw: Saturated, increase size of CGW_TRN_MAX!");
-                list_produce(privdata->outgoing_queue, msg);
+                gwlist_produce(privdata->outgoing_queue, msg);
                 return 1;     /* re-insert, and go check for acks */
             }
         }
@@ -739,7 +798,7 @@ void cgw_check_acks(PrivData *privdata)
                 privdata->unacked--;
                 warning(0, "smsc_cgw: received neither OK nor ERR for message %d "
                         "in %d seconds, resending message", i, privdata->waitack);
-                list_produce(privdata->outgoing_queue, privdata->sendmsg[i]);
+                gwlist_produce(privdata->outgoing_queue, privdata->sendmsg[i]);
             }
     }
 }
@@ -768,7 +827,7 @@ int cgw_wait_command(PrivData *privdata, SMSCConn *conn, Connection *server, int
 		    if (cgwop != NULL) cgwop_destroy(cgwop);
 		    return -1;
 		}
-		if (conn_read_error(server)) {
+		if (conn_error(server)) {
 		    error(0, "cgw: Error trying to read ACKs from SMSC");
 		    if (cgwop != NULL) cgwop_destroy(cgwop);
 		    return -1;
@@ -858,10 +917,10 @@ struct cgwop *cgw_read_op(PrivData *privdata, SMSCConn *conn, Connection *server
             if (op != CGW_OP_NOP) {
                 /* All commands have to be inside an op:xx ... end:xx statement */
 
-                if (octstr_compare(name, octstr_imm("end")) == 0) { // found end of op
+                if (octstr_compare(name, octstr_imm("end")) == 0) { /* found end of op */
                     finished = 1;
                 } else {
-                    // store in name/value fields in cgwop
+                    /* store in name/value fields in cgwop */
                     if (cgwop != NULL) {
                         cgwop_add(cgwop, name, value);
                     }
@@ -883,12 +942,11 @@ struct cgwop *cgw_read_op(PrivData *privdata, SMSCConn *conn, Connection *server
     return cgwop;
 }
 
-static int cgw_open_listening_socket(PrivData *privdata)
+static int cgw_open_listening_socket(SMSCConn *conn, PrivData *privdata)
 {
     int s;
 
-    if ((s = make_server_socket(privdata->rport, NULL)) == -1) {
-        /* XXX add interface_name if required */
+    if ((s = make_server_socket(privdata->rport, (conn->our_host ? octstr_get_cstr(conn->our_host) : NULL))) == -1) {
         error(0, "smsc_cgw: could not create listening socket in port %d", privdata->rport);
         return -1;
     }
@@ -916,6 +974,9 @@ static void cgw_listener(void *arg)
     Connection	*server;
     int s, ret;
 
+    /* Make sure we log into our own log-file if defined */
+    log_thread_to(conn->log_idx);
+
     while (!privdata->shutdown) {
         server_addr_len = sizeof(server_addr);
 	
@@ -976,7 +1037,7 @@ static void cgw_receiver(SMSCConn *conn, Connection *server)
             info(0, "cgw: receive connection closed by SMSC");
             return ;
         }
-        if (conn_read_error(server)) {
+        if (conn_error(server)) {
             error(0, "cgw: receive connection broken");
             return ;
         }
@@ -1052,7 +1113,7 @@ static int cgw_handle_op(SMSCConn *conn, Connection *server, struct cgwop *cgwop
 
         reply = cgwop_create(CGW_OP_OK, -1);
         cgwop_add(reply, octstr_imm("session-id"), sid);
-        cgwop_send(server, reply);     // send reply
+        cgwop_send(server, reply);     /* send reply */
 
         cgwop_destroy(reply);
 
@@ -1073,28 +1134,27 @@ static int cgw_handle_op(SMSCConn *conn, Connection *server, struct cgwop *cgwop
 
             switch (stat) {
             case 0:     /* delivered */
-                dlrmsg = dlr_find(octstr_get_cstr(conn->id),
-                                  octstr_get_cstr(ts),     /* timestamp */
-                                  octstr_get_cstr(msid),   /* destination */
+                dlrmsg = dlr_find(conn->id,
+                                            ts,     /* timestamp */
+                                            msid,   /* destination */
                                   DLR_SUCCESS);
                 break;
             case 1:     /* buffered */
-                dlrmsg = dlr_find(octstr_get_cstr(conn->id),
-                                  octstr_get_cstr(ts),     /* timestamp */
-                                  octstr_get_cstr(msid),   /* destination */
+                dlrmsg = dlr_find(conn->id,
+                                            ts,     /* timestamp */
+                                            msid,   /* destination */
                                   DLR_BUFFERED);
                 break;
             case 2:     /* not delivered */
-                dlrmsg = dlr_find(octstr_get_cstr(conn->id),
-                                  octstr_get_cstr(ts),     /* timestamp */
-                                  octstr_get_cstr(msid),   /* destination */
+                dlrmsg = dlr_find(conn->id,
+                                            ts,     /* timestamp */
+                                            msid,   /* destination */
                                   DLR_FAIL);
                 break;
             }
 
             octstr_destroy(ts);
             if (dlrmsg != NULL) {
-
                 dlrmsg->sms.msgdata = octstr_duplicate(txt);
                 bb_smscconn_receive(conn, dlrmsg);
             }
@@ -1113,7 +1173,7 @@ static int cgw_handle_op(SMSCConn *conn, Connection *server, struct cgwop *cgwop
 
         msg = privdata->sendmsg[trn];
 
-        if (msg && msg->sms.dlr_url && (msg->sms.dlr_mask & 0x07)) {
+        if (msg && msg->sms.dlr_url && DLR_IS_ENABLED_DEVICE(msg->sms.dlr_mask)) {
             Octstr *ts;
 
             ts = octstr_create("");
@@ -1121,13 +1181,7 @@ static int cgw_handle_op(SMSCConn *conn, Connection *server, struct cgwop *cgwop
             octstr_append_char(ts, '-');
             octstr_append_decimal(ts, trn);
 
-            dlr_add(octstr_get_cstr(conn->id),
-                    octstr_get_cstr(ts),
-                    octstr_get_cstr(msg->sms.sender),
-                    octstr_get_cstr(msg->sms.receiver),
-                    octstr_get_cstr(msg->sms.service),
-                    octstr_get_cstr(msg->sms.dlr_url),
-                    msg->sms.dlr_mask);
+            dlr_add(conn->id, ts, msg);
 
             octstr_destroy(ts);
             privdata->dlr[trn] = 1;
@@ -1135,7 +1189,8 @@ static int cgw_handle_op(SMSCConn *conn, Connection *server, struct cgwop *cgwop
             privdata->dlr[trn] = 0;
         }
 
-        bb_smscconn_sent(conn, msg);     /* mark as successfully sent */
+	/* mark as successfully sent */
+        bb_smscconn_sent(conn, msg, NULL);
 
         break;
 
@@ -1161,7 +1216,7 @@ static int cgw_handle_op(SMSCConn *conn, Connection *server, struct cgwop *cgwop
         privdata->unacked--;
 
         bb_smscconn_send_failed(conn, privdata->sendmsg[trn],
-                                SMSCCONN_FAILED_REJECTED);
+                            SMSCCONN_FAILED_REJECTED, octstr_create("REJECTED"));
 
         break;
 
