@@ -1,7 +1,7 @@
 /* ==================================================================== 
  * The Kannel Software License, Version 1.0 
  * 
- * Copyright (c) 2001-2004 Kannel Group  
+ * Copyright (c) 2001-2005 Kannel Group  
  * Copyright (c) 1998-2001 WapIT Ltd.   
  * All rights reserved. 
  * 
@@ -992,7 +992,7 @@ static int emi2_do_send(SMSCConn *conn, Connection *server)
     Msg *msg;
     double delay = 0;
 
-    if (conn->throughput) {
+    if (conn->throughput > 0) {
         delay = 1.0 / conn->throughput;
     }
     
@@ -1001,7 +1001,7 @@ static int emi2_do_send(SMSCConn *conn, Connection *server)
            (msg = gw_prioqueue_remove(PRIVDATA(conn)->outgoing_queue)) != NULL) {
         int nexttrn = emi2_next_trn(conn);
 
-        if (conn->throughput)
+        if (conn->throughput > 0)
             gwthread_sleep(delay);
 
         /* convert the generic Kannel message into an EMI type message */
