@@ -1,3 +1,59 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2005 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*
  * urltrans.c - URL translations
  *
@@ -15,6 +71,7 @@
 #include "urltrans.h"
 #include "gwlib/gwlib.h"
 #include "gw/sms.h"
+#include "gwlib/regex.h"
 
 
 /***********************************************************************
@@ -70,6 +127,15 @@ struct URLTranslation {
     int has_catchall_arg;
     int catch_all;
     Octstr *dlr_url;	/* Url to call for delivery reports */
+
+    regex_t *keyword_regex;       /* the compiled regular expression for the keyword*/
+    regex_t *accepted_smsc_regex;
+    regex_t *allowed_prefix_regex;
+    regex_t *denied_prefix_regex;
+    regex_t *allowed_receiver_prefix_regex;
+    regex_t *denied_receiver_prefix_regex;
+    regex_t *white_list_regex;
+    regex_t *black_list_regex;
 };
 
 
@@ -109,7 +175,7 @@ static URLTranslation *find_black_list_translation(URLTranslationList *trans,
 
 static void destroy_keyword_list(void *list)
 {
-    list_destroy(list, NULL);
+    gwlist_destroy(list, NULL);
 }
 
 
@@ -118,7 +184,7 @@ URLTranslationList *urltrans_create(void)
     URLTranslationList *trans;
     
     trans = gw_malloc(sizeof(URLTranslationList));
-    trans->list = list_create();
+    trans->list = gwlist_create();
     trans->dict = dict_create(1024, destroy_keyword_list);
     trans->names = dict_create(1024, destroy_keyword_list);
     return trans;
@@ -127,7 +193,7 @@ URLTranslationList *urltrans_create(void)
 
 void urltrans_destroy(URLTranslationList *trans) 
 {
-    list_destroy(trans->list, destroy_onetrans);
+    gwlist_destroy(trans->list, destroy_onetrans);
     dict_destroy(trans->names);
     dict_destroy(trans->dict);
     gw_free(trans);
@@ -145,33 +211,33 @@ int urltrans_add_one(URLTranslationList *trans, CfgGroup *grp)
     if (ot == NULL)
 	return -1;
 		
-    list_append(trans->list, ot);
+    gwlist_append(trans->list, ot);
     
     list2 = dict_get(trans->names, ot->name);
     if (list2 == NULL) {
-    	list2 = list_create();
+    	list2 = gwlist_create();
 	dict_put(trans->names, ot->name, list2);
     }
-    list_append(list2, ot);
+    gwlist_append(list2, ot);
 
     if (ot->keyword == NULL || ot->type == TRANSTYPE_SENDSMS)
     	return 0;
 
     list = dict_get(trans->dict, ot->keyword);
     if (list == NULL) {
-    	list = list_create();
+    	list = gwlist_create();
 	dict_put(trans->dict, ot->keyword, list);
     }
-    list_append(list, ot);
+    gwlist_append(list, ot);
 
-    for (i = 0; i < list_len(ot->aliases); ++i) {
-	alias = list_get(ot->aliases, i);
+    for (i = 0; i < gwlist_len(ot->aliases); ++i) {
+	alias = gwlist_get(ot->aliases, i);
 	list = dict_get(trans->dict, alias);
 	if (list == NULL) {
-	    list = list_create();
+	    list = gwlist_create();
 	    dict_put(trans->dict, alias, list);
 	}
-	list_append(list, ot);
+	gwlist_append(list, ot);
     }
 
 
@@ -185,22 +251,22 @@ int urltrans_add_cfg(URLTranslationList *trans, Cfg *cfg)
     List *list;
     
     list = cfg_get_multi_group(cfg, octstr_imm("sms-service"));
-    while (list && (grp = list_extract_first(list)) != NULL) {
+    while (list && (grp = gwlist_extract_first(list)) != NULL) {
 	if (urltrans_add_one(trans, grp) == -1) {
-	    list_destroy(list, NULL);
+	    gwlist_destroy(list, NULL);
 	    return -1;
 	}
     }
-    list_destroy(list, NULL);
+    gwlist_destroy(list, NULL);
     
     list = cfg_get_multi_group(cfg, octstr_imm("sendsms-user"));
-    while (list && (grp = list_extract_first(list)) != NULL) {
+    while (list && (grp = gwlist_extract_first(list)) != NULL) {
 	if (urltrans_add_one(trans, grp) == -1) {
-	    list_destroy(list, NULL);
+	    gwlist_destroy(list, NULL);
 	    return -1;
 	}
     }
-    list_destroy(list, NULL);
+    gwlist_destroy(list, NULL);
 
     return 0;
 }
@@ -210,13 +276,16 @@ URLTranslation *urltrans_find(URLTranslationList *trans, Octstr *text,
 			      Octstr *smsc, Octstr *sender, Octstr *receiver) 
 {
     List *words;
-    URLTranslation *t;
+    URLTranslation *t = NULL;
     int reject = 0;
     
-    words = octstr_split_words(text);
+    /* do not panic if text == NULL */
+    if (text != NULL) {
+        words = octstr_split_words(text);
+        t = find_translation(trans, words, smsc, sender, receiver, &reject);
+        gwlist_destroy(words, octstr_destroy_item);
+    }
     
-    t = find_translation(trans, words, smsc, sender, receiver, &reject);
-    list_destroy(words, octstr_destroy_item);
     if (reject)
 	t = find_black_list_translation(trans, smsc);
     if (t == NULL) {
@@ -235,7 +304,7 @@ URLTranslation *urltrans_find_service(URLTranslationList *trans, Msg *msg)
     
     list = dict_get(trans->names, msg->sms.service);
     if (list != NULL) {
-       t = list_get(list, 0);
+       t = gwlist_get(list, 0);
     } else  {
        t = NULL;
     }
@@ -251,8 +320,8 @@ URLTranslation *urltrans_find_username(URLTranslationList *trans,
     int i;
 
     gw_assert(name != NULL);
-    for (i = 0; i < list_len(trans->list); ++i) {
-	t = list_get(trans->list, i);
+    for (i = 0; i < gwlist_len(trans->list); ++i) {
+	t = gwlist_get(trans->list, i);
 	if (t->type == TRANSTYPE_SENDSMS) {
 	    if (octstr_compare(name, t->username) == 0)
 		return t;
@@ -284,7 +353,10 @@ static void strip_keyword(Msg *request)
 }
 
 
-
+/*
+ * Trans being NULL means that we are servicing ppg (doing dlr, but this does not
+ * concern us here).
+ */
 Octstr *urltrans_get_pattern(URLTranslation *t, Msg *request)
 {
     Octstr *enc;
@@ -302,16 +374,21 @@ Octstr *urltrans_get_pattern(URLTranslation *t, Msg *request)
 
     url = reply = NULL;
     
-    if (request->sms.sms_type != report && t->type == TRANSTYPE_SENDSMS)
+    if (request->sms.sms_type != report_mo && t->type == TRANSTYPE_SENDSMS)
         return octstr_create("");
 
-    word_list = octstr_split_words(request->sms.msgdata);
-    num_words = list_len(word_list);
+    if (request->sms.msgdata) {
+        word_list = octstr_split_words(request->sms.msgdata);
+        num_words = gwlist_len(word_list);
+    } else {
+    	word_list = gwlist_create();
+        num_words = 0;
+    }
 
     result = octstr_create("");
 
     /* check if this is a delivery report message or not */
-    if (request->sms.sms_type != report) {
+    if (request->sms.sms_type != report_mo) {
         pattern = t->pattern;
     } else {
 
@@ -321,10 +398,10 @@ Octstr *urltrans_get_pattern(URLTranslation *t, Msg *request)
 
         pattern = url;
         if (octstr_len(pattern) == 0) {
-            if (octstr_len(t->dlr_url)) {
+            if (t && octstr_len(t->dlr_url)) {
                 pattern = t->dlr_url;
             } else {
-                list_destroy(word_list, octstr_destroy_item);
+                gwlist_destroy(word_list, octstr_destroy_item);
                 return octstr_create("");
             }
         }
@@ -346,10 +423,10 @@ Octstr *urltrans_get_pattern(URLTranslation *t, Msg *request)
             break;
 
     switch (octstr_get_char(pattern, pos + 1)) {
-    case 'k':
+	case 'k':
 	    if (num_words <= 0)
 		break;
-	    enc = octstr_duplicate(list_get(word_list, 0));
+	    enc = octstr_duplicate(gwlist_get(word_list, 0));
 	    octstr_url_encode(enc);
 	    octstr_append(result, enc);
 	    octstr_destroy(enc);
@@ -358,7 +435,7 @@ Octstr *urltrans_get_pattern(URLTranslation *t, Msg *request)
 	case 's':
 	    if (nextarg >= num_words)
 		break;
-	    enc = octstr_duplicate(list_get(word_list, nextarg));
+	    enc = octstr_duplicate(gwlist_get(word_list, nextarg));
 	    octstr_url_encode(enc);
 	    octstr_append(result, enc);
 	    octstr_destroy(enc);
@@ -368,7 +445,7 @@ Octstr *urltrans_get_pattern(URLTranslation *t, Msg *request)
 	case 'S':
 	    if (nextarg >= num_words)
 		break;
-	    temp = list_get(word_list, nextarg);
+	    temp = gwlist_get(word_list, nextarg);
 	    for (i = 0; i < octstr_len(temp); ++i) {
 		if (octstr_get_char(temp, i) == '*')
 		    octstr_append_char(result, '~');
@@ -380,7 +457,7 @@ Octstr *urltrans_get_pattern(URLTranslation *t, Msg *request)
 
 	case 'r':
 	    for (j = nextarg; j < num_words; ++j) {
-		enc = octstr_duplicate(list_get(word_list, j));
+		enc = octstr_duplicate(gwlist_get(word_list, j));
 		octstr_url_encode(enc);
 		if (j != nextarg)
 		    octstr_append_char(result, '+');
@@ -439,7 +516,7 @@ Octstr *urltrans_get_pattern(URLTranslation *t, Msg *request)
 
 	case 'a':
 	    for (j = 0; j < num_words; ++j) {
-		enc = octstr_duplicate(list_get(word_list, j));
+		enc = octstr_duplicate(gwlist_get(word_list, j));
 		octstr_url_encode(enc);
 		if (j > 0)
 		    octstr_append_char(result, '+');
@@ -466,6 +543,12 @@ Octstr *urltrans_get_pattern(URLTranslation *t, Msg *request)
 				 tm.tm_sec);
 	    break;
 
+	case 'T':
+	    if (request->sms.time == MSG_PARAM_UNDEFINED)
+		break;
+	    octstr_format_append(result, "%ld", request->sms.time);
+	    break;
+
 	case 'i':
 	    if (request->sms.smsc_id == NULL)
 		break;
@@ -475,6 +558,14 @@ Octstr *urltrans_get_pattern(URLTranslation *t, Msg *request)
 	    octstr_destroy(enc);
 	    break;
 
+	case 'I':
+	    if (!uuid_is_null(request->sms.id)) {
+                char id[UUID_STR_LEN + 1];
+                uuid_unparse(request->sms.id, id);
+	        octstr_append_cstr(result, id);
+            }
+	    break;
+
 	case 'n':
 	    if (request->sms.service == NULL)
 		break;
@@ -493,10 +584,12 @@ Octstr *urltrans_get_pattern(URLTranslation *t, Msg *request)
 	    break;
 
 	case 'A':
-	    enc = octstr_duplicate(reply);
-	    octstr_url_encode(enc);
-	    octstr_append(result, enc);
-	    octstr_destroy(enc);
+	    if (reply) {
+		enc = octstr_duplicate(reply);
+		octstr_url_encode(enc);
+		octstr_append(result, enc);
+		octstr_destroy(enc);
+	    }
 	    break;
 
 	case 'c':
@@ -510,13 +603,13 @@ Octstr *urltrans_get_pattern(URLTranslation *t, Msg *request)
 		switch (request->sms.coding) {
 		    case DC_UNDEF:
 		    case DC_7BIT:
-			octstr_append(result, octstr_imm("gsm"));
+			octstr_append(result, octstr_imm("ISO-8859-1"));
 			break;
 		    case DC_8BIT:
-			octstr_append(result, octstr_imm("binary"));
+			octstr_append(result, octstr_imm("8-BIT"));
 			break;
 		    case DC_UCS2:
-			octstr_append(result, octstr_imm("UTF16-BE"));
+			octstr_append(result, octstr_imm("UTF-16BE"));
 			break;
 		}
 	    }
@@ -531,6 +624,59 @@ Octstr *urltrans_get_pattern(URLTranslation *t, Msg *request)
 	    }
 	    break;
 
+	case 'B':  /* billing identifier/information */
+	    if (octstr_len(request->sms.binfo)) {
+            enc = octstr_duplicate(request->sms.binfo);
+            octstr_url_encode(enc);
+            octstr_append(result, enc);
+            octstr_destroy(enc);
+        }
+        break;
+	
+    case 'o':  /* account information (may be operator id for aggregators */
+	    if (octstr_len(request->sms.account)) {
+            enc = octstr_duplicate(request->sms.account);
+            octstr_url_encode(enc);
+            octstr_append(result, enc);
+            octstr_destroy(enc);
+        }
+        break;
+
+    case 'm':  /* mclass - message class */
+        enc = octstr_create("");
+        octstr_append_decimal(enc, request->sms.mclass);
+        octstr_url_encode(enc);
+        octstr_append(result, enc);
+        octstr_destroy(enc);
+        break;
+
+    case 'M':  /* mwi - message waiting indicator */
+        enc = octstr_create("");
+        octstr_append_decimal(enc, request->sms.mwi);
+        octstr_url_encode(enc);
+        octstr_append(result, enc);
+        octstr_destroy(enc);
+        break;
+
+    case 'f':  /* smsc number*/
+	    if (octstr_len(request->sms.smsc_number)) {
+            enc = octstr_duplicate(request->sms.smsc_number);
+            octstr_url_encode(enc);
+            octstr_append(result, enc);
+            octstr_destroy(enc);
+        }
+        break;
+
+	/* XXX sms.parameters not present in here:
+	 *   * pid - will we receive this ? 
+	 *   * alt-dcs - shouldn't be required unless we want to inform 
+	 *               which alt-dcs external server should use back
+	 *   * compress - if we use compression, probably kannel would 
+	 *                decompress and reset this to 0. not required
+	 *   * validity, deferred, rpi - we don't receive these from smsc
+	 *   * username, password, dlr-url, account - nonsense to send
+	 */
+
 	case '%':
 	    octstr_format_append(result, "%%");
 	    break;
@@ -547,14 +693,14 @@ Octstr *urltrans_get_pattern(URLTranslation *t, Msg *request)
      * this SHOULD be done in smsbox, not here, but well,
      * much easier to do here
      */
-    if ( (t->type == TRANSTYPE_POST_URL || t->type == TRANSTYPE_POST_XML)
+    if (t && (t->type == TRANSTYPE_POST_URL || t->type == TRANSTYPE_POST_XML)
 		    && t->strip_keyword)
 	strip_keyword(request);
 
     octstr_destroy(url);
     octstr_destroy(reply);
 
-    list_destroy(word_list, octstr_destroy_item);
+    gwlist_destroy(word_list, octstr_destroy_item);
     return result;
 }
 
@@ -679,11 +825,21 @@ Numhash *urltrans_white_list(URLTranslation *t)
     return t->white_list;
 }
 
+regex_t *urltrans_white_list_regex(URLTranslation *t)
+{
+    return t->white_list_regex;
+}
+
 Numhash *urltrans_black_list(URLTranslation *t)
 {
     return t->black_list;
 }
 
+regex_t *urltrans_black_list_regex(URLTranslation *t)
+{
+    return t->black_list_regex;
+}
+
 int urltrans_assume_plain_text(URLTranslation *t) 
 {
     return t->assume_plain_text;
@@ -721,6 +877,15 @@ static URLTranslation *create_onetrans(CfgGroup *grp)
     Octstr *accepted_smsc, *forced_smsc, *default_smsc;
     Octstr *grpname, *sendsms_user, *sms_service;
     int is_sms_service;
+    Octstr *accepted_smsc_regex;
+    Octstr *allowed_prefix_regex;
+    Octstr *denied_prefix_regex;
+    Octstr *allowed_receiver_prefix_regex;
+    Octstr *denied_receiver_prefix_regex;
+    Octstr *white_list_regex;
+    Octstr *black_list_regex;
+    Octstr *keyword_regex;
+    Octstr *os;
     
     grpname = cfg_get_group_name(grp);
     if (grpname == NULL)
@@ -766,6 +931,14 @@ static URLTranslation *create_onetrans(CfgGroup *grp)
     ot->denied_recv_prefix = NULL;
     ot->white_list = NULL;
     ot->black_list = NULL;
+    ot->keyword_regex = NULL;
+    ot->accepted_smsc_regex = NULL;
+    ot->allowed_prefix_regex = NULL;
+    ot->denied_prefix_regex = NULL;
+    ot->allowed_receiver_prefix_regex = NULL;
+    ot->denied_receiver_prefix_regex = NULL;
+    ot->white_list_regex = NULL;
+    ot->black_list_regex = NULL;
     
     if (is_sms_service) {
 	cfg_get_bool(&ot->catch_all, grp, octstr_imm("catch-all"));
@@ -815,21 +988,28 @@ static URLTranslation *create_onetrans(CfgGroup *grp)
 	octstr_convert_range(ot->keyword, 0, octstr_len(ot->keyword), 
 	    	    	     tolower);
 
+        keyword_regex = cfg_get(grp, octstr_imm("keyword-regex"));
+        if (keyword_regex != NULL) {
+            if ((ot->keyword_regex = gw_regex_comp(keyword_regex, REG_EXTENDED)) == NULL)
+                  panic(0, "Could not compile pattern '%s'", octstr_get_cstr(keyword_regex));
+            octstr_destroy(keyword_regex);
+        }
+
 	ot->name = cfg_get(grp, octstr_imm("name"));
 	if (ot->name == NULL)
 	    ot->name = octstr_duplicate(ot->keyword);
 
 	aliases = cfg_get(grp, octstr_imm("aliases"));
 	if (aliases == NULL)
-	    ot->aliases = list_create();
+	    ot->aliases = gwlist_create();
 	else {
 	    long i;
 	    Octstr *os;
 
 	    ot->aliases = octstr_split(aliases, octstr_imm(";"));
 	    octstr_destroy(aliases);
-	    for (i = 0; i < list_len(ot->aliases); ++i) {
-		os = list_get(ot->aliases, i);
+	    for (i = 0; i < gwlist_len(ot->aliases); ++i) {
+		os = gwlist_get(ot->aliases, i);
 	    	octstr_convert_range(os, 0, octstr_len(os), tolower);
 	    }
 	}
@@ -839,6 +1019,12 @@ static URLTranslation *create_onetrans(CfgGroup *grp)
 	    ot->accepted_smsc = octstr_split(accepted_smsc, octstr_imm(";"));
 	    octstr_destroy(accepted_smsc);
 	}
+        accepted_smsc_regex = cfg_get(grp, octstr_imm("accepted-smsc-regex"));
+        if (accepted_smsc_regex != NULL) { 
+            if ( (ot->accepted_smsc_regex = gw_regex_comp(accepted_smsc_regex, REG_EXTENDED)) == NULL)
+            panic(0, "Could not compile pattern '%s'", octstr_get_cstr(accepted_smsc_regex));
+            octstr_destroy(accepted_smsc_regex);
+        }
 
 	cfg_get_bool(&ot->assume_plain_text, grp, 
 		     octstr_imm("assume-plain-text"));
@@ -849,9 +1035,22 @@ static URLTranslation *create_onetrans(CfgGroup *grp)
 	
 	ot->prefix = cfg_get(grp, octstr_imm("prefix"));
 	ot->suffix = cfg_get(grp, octstr_imm("suffix"));
+        ot->allowed_recv_prefix = cfg_get(grp, octstr_imm("allowed-receiver-prefix"));
+        allowed_receiver_prefix_regex = cfg_get(grp, octstr_imm("allowed-receiver-prefix-regex"));
+        if (allowed_receiver_prefix_regex != NULL) {
+            if ((ot->allowed_receiver_prefix_regex = gw_regex_comp(allowed_receiver_prefix_regex, REG_EXTENDED)) == NULL)
+            panic(0, "Could not compile pattern '%s'", octstr_get_cstr(allowed_receiver_prefix_regex));
+            octstr_destroy(allowed_receiver_prefix_regex);
+        }
 
 	ot->allowed_recv_prefix = cfg_get(grp, octstr_imm("allowed-receiver-prefix"));
     ot->denied_recv_prefix = cfg_get(grp, octstr_imm("denied-receiver-prefix"));
+        denied_receiver_prefix_regex = cfg_get(grp, octstr_imm("denied-receiver-prefix-regex"));
+        if (denied_receiver_prefix_regex != NULL) {
+            if ((ot->denied_receiver_prefix_regex = gw_regex_comp(denied_receiver_prefix_regex, REG_EXTENDED)) == NULL)
+            panic(0, "Could not compile pattern '%s'",octstr_get_cstr(denied_receiver_prefix_regex));
+            octstr_destroy(denied_receiver_prefix_regex);
+        }
 
 	ot->args = count_occurences(ot->pattern, octstr_imm("%s"));
 	ot->args += count_occurences(ot->pattern, octstr_imm("%S"));
@@ -891,22 +1090,45 @@ static URLTranslation *create_onetrans(CfgGroup *grp)
 	ot->deny_ip = cfg_get(grp, octstr_imm("user-deny-ip"));
 	ot->allow_ip = cfg_get(grp, octstr_imm("user-allow-ip"));
 	ot->default_sender = cfg_get(grp, octstr_imm("default-sender"));
-
     }
+    
     ot->allowed_prefix = cfg_get(grp, octstr_imm("allowed-prefix"));
+    allowed_prefix_regex = cfg_get(grp, octstr_imm("allowed-prefix-regex"));
+    if (allowed_prefix_regex != NULL) {
+        if ((ot->allowed_prefix_regex = gw_regex_comp(allowed_prefix_regex, REG_EXTENDED)) == NULL)
+            panic(0, "Could not compile pattern '%s'", octstr_get_cstr(allowed_prefix_regex));
+        octstr_destroy(allowed_prefix_regex);
+    }
     ot->denied_prefix = cfg_get(grp, octstr_imm("denied-prefix"));
-    {
-	Octstr *os;
+    denied_prefix_regex = cfg_get(grp, octstr_imm("denied-prefix-regex"));
+    if (denied_prefix_regex != NULL) {
+        if ((ot->denied_prefix_regex = gw_regex_comp(denied_prefix_regex, REG_EXTENDED)) == NULL)
+            panic(0, "Could not compile pattern '%s'", octstr_get_cstr(denied_prefix_regex));
+        octstr_destroy(denied_prefix_regex);
+    }
+    
 	os = cfg_get(grp, octstr_imm("white-list"));
 	if (os != NULL) {
 	    ot->white_list = numhash_create(octstr_get_cstr(os));
 	    octstr_destroy(os);
 	}
+    white_list_regex = cfg_get(grp, octstr_imm("white-list-regex"));
+    if (white_list_regex != NULL) {
+        if ((ot->white_list_regex = gw_regex_comp(white_list_regex, REG_EXTENDED)) == NULL)
+            panic(0, "Could not compile pattern '%s'", octstr_get_cstr(white_list_regex));
+        octstr_destroy(white_list_regex);
+    }
+
 	os = cfg_get(grp, octstr_imm("black-list"));
 	if (os != NULL) {
 	    ot->black_list = numhash_create(octstr_get_cstr(os));
 	    octstr_destroy(os);
 	}
+    black_list_regex = cfg_get(grp, octstr_imm("black-list-regex"));
+    if (black_list_regex != NULL) {
+        if ((ot->black_list_regex = gw_regex_comp(black_list_regex, REG_EXTENDED)) == NULL)
+            panic(0, "Could not compile pattern '%s'", octstr_get_cstr(black_list_regex));
+        octstr_destroy(black_list_regex);
     }
 
     if (cfg_get_integer(&ot->max_messages, grp, 
@@ -952,8 +1174,8 @@ static void destroy_onetrans(void *p)
     
     ot = p;
     if (ot != NULL) {
-	octstr_destroy(ot->keyword);
-	list_destroy(ot->aliases, octstr_destroy_item);
+        octstr_destroy(ot->keyword);
+	gwlist_destroy(ot->aliases, octstr_destroy_item);
 	octstr_destroy(ot->dlr_url);
 	octstr_destroy(ot->pattern);
 	octstr_destroy(ot->prefix);
@@ -964,7 +1186,7 @@ static void destroy_onetrans(void *p)
 	octstr_destroy(ot->split_suffix);
 	octstr_destroy(ot->header);
 	octstr_destroy(ot->footer);
-	list_destroy(ot->accepted_smsc, octstr_destroy_item);
+	gwlist_destroy(ot->accepted_smsc, octstr_destroy_item);
 	octstr_destroy(ot->name);
 	octstr_destroy(ot->username);
 	octstr_destroy(ot->password);
@@ -978,98 +1200,199 @@ static void destroy_onetrans(void *p)
 	octstr_destroy(ot->denied_recv_prefix);
 	numhash_destroy(ot->white_list);
 	numhash_destroy(ot->black_list);
+        if (ot->keyword_regex != NULL) gw_regex_destroy(ot->keyword_regex);
+        if (ot->accepted_smsc_regex != NULL) gw_regex_destroy(ot->accepted_smsc_regex);
+        if (ot->allowed_prefix_regex != NULL) gw_regex_destroy(ot->allowed_prefix_regex);
+        if (ot->denied_prefix_regex != NULL) gw_regex_destroy(ot->denied_prefix_regex);
+        if (ot->allowed_receiver_prefix_regex != NULL) gw_regex_destroy(ot->allowed_receiver_prefix_regex);
+        if (ot->denied_receiver_prefix_regex != NULL) gw_regex_destroy(ot->denied_receiver_prefix_regex);
+        if (ot->white_list_regex != NULL) gw_regex_destroy(ot->white_list_regex);
+        if (ot->black_list_regex != NULL) gw_regex_destroy(ot->black_list_regex);
 	gw_free(ot);
     }
 }
 
 
+/*
+ * checks if the number of passed words matches the service-pattern defined in the
+ * translation. returns 0 if arguments are okay, -1 otherwise.
+ */
+static int check_num_args(URLTranslation *t, List *words)
+{
+    const int IS_OKAY = 0;
+    const int NOT_OKAY = -1;
+    int n;
+
+    
+    n = gwlist_len(words);
+    /* check number of arguments */
+    if (t->catch_all)
+        return IS_OKAY;
+    
+    if (n - 1 == t->args)
+        return IS_OKAY;
+
+    if (t->has_catchall_arg && n - 1 >= t->args)
+        return IS_OKAY;
+
+    return NOT_OKAY;
+}
+
+/*
+ * checks if a request matches the parameters of a URL-Translation, e.g. whether or not 
+ * a user is allowed to use certain services. returns 0 if allowed, -1 if not.
+ * reject will be set to 1 is a number is rejected due to white/black-lists.
+ */
+static int check_allowed_translation(URLTranslation *t, 
+                  Octstr *smsc, Octstr *sender, Octstr *receiver, int *reject)
+{
+    const int IS_ALLOWED = 0;
+    const int NOT_ALLOWED = -1;
+
+    /* if smsc_id set and accepted_smsc exist, accept
+     * translation only if smsc id is in accept string
+     */
+    if (smsc && t->accepted_smsc && !gwlist_search(t->accepted_smsc, smsc, octstr_item_match))
+        return NOT_ALLOWED;
+
+    if (smsc && t->accepted_smsc_regex && gw_regex_match_pre( t->accepted_smsc_regex, smsc) == 0)
+        return NOT_ALLOWED;
+
+    /* Have allowed for sender */
+    if (t->allowed_prefix && !t->denied_prefix && does_prefix_match(t->allowed_prefix, sender) != 1)
+        return NOT_ALLOWED;
+
+    if (t->allowed_prefix_regex && !t->denied_prefix_regex && gw_regex_match_pre(t->allowed_prefix_regex, sender) == 0)
+        return NOT_ALLOWED;
+
+    /* Have denied for sender */
+    if (t->denied_prefix && !t->allowed_prefix && does_prefix_match(t->denied_prefix, sender) == 1)
+        return NOT_ALLOWED;
+
+    if (t->denied_prefix_regex && !t->allowed_prefix_regex && gw_regex_match_pre(t->denied_prefix_regex, sender) == 1)
+        return NOT_ALLOWED;
+
+    /* Have allowed for receiver */
+    if (t->allowed_recv_prefix && !t->denied_recv_prefix && does_prefix_match(t->allowed_recv_prefix, receiver) != 1)
+        return NOT_ALLOWED;
+
+    if (t->allowed_receiver_prefix_regex && !t->denied_receiver_prefix_regex &&
+        gw_regex_match_pre(t->allowed_receiver_prefix_regex, receiver) == 0)
+        return NOT_ALLOWED;
+
+    /* Have denied for receiver */
+    if (t->denied_recv_prefix && !t->allowed_recv_prefix && does_prefix_match(t->denied_recv_prefix, receiver) == 1)
+        return NOT_ALLOWED;
+
+    if (t->denied_receiver_prefix_regex && !t->allowed_receiver_prefix_regex &&
+        gw_regex_match_pre(t->denied_receiver_prefix_regex, receiver) == 0)
+        return NOT_ALLOWED;
+
+    if (t->white_list && numhash_find_number(t->white_list, sender) < 1) {
+        *reject = 1;
+        return NOT_ALLOWED;
+    }
+
+    if (t->white_list_regex && gw_regex_match_pre(t->white_list_regex, sender) == 0) {
+        *reject = 1;
+        return NOT_ALLOWED;
+    }   
+
+    if (t->black_list && numhash_find_number(t->black_list, sender) == 1) {
+        *reject = 1;
+        return NOT_ALLOWED;
+    }
+
+    if (t->black_list_regex && gw_regex_match_pre(t->black_list_regex, sender) == 1) {
+        *reject = 1;
+        return NOT_ALLOWED;
+    }   
+
+    /* Have allowed and denied */
+    if (t->denied_prefix && t->allowed_prefix && does_prefix_match(t->allowed_prefix, sender) != 1 &&
+        does_prefix_match(t->denied_prefix, sender) == 1)
+        return NOT_ALLOWED;
+
+    if (t->denied_prefix_regex && t->allowed_prefix_regex &&
+        gw_regex_match_pre(t->allowed_prefix_regex, sender) == 0 &&
+        gw_regex_match_pre(t->denied_prefix_regex, sender) == 1)
+        return NOT_ALLOWED;
+
+    return IS_ALLOWED;
+};
+
+    
+/* get_matching_translations - iterate over all translations in trans. 
+ * for each translation check whether 
+ * the translation's keyword has already been interpreted as a regexp. 
+ * if not, compile it now,
+ * otherwise retrieve compilation result from dictionary.
+ *
+ * the translations where the word matches the translation's pattern 
+ * are returned in a list
+ * 
+ */
+static List* get_matching_translations(URLTranslationList *trans, Octstr *word) 
+{
+    List *list;
+    /*char *tmp_word;*/
+    int i;
+    size_t n_match = 1;
+    regmatch_t p_match[10];
+    URLTranslation *t;
+
+    gw_assert(trans != NULL && word != NULL);
+
+    list = gwlist_create();
+    for (i = 0; i < gwlist_len(trans->list); ++i) {
+        t = gwlist_get(trans->list, i);
+        if (t->keyword == NULL) 
+	    continue;
+
+        /* if regex feature is used try to match */
+        if ((t->keyword_regex != NULL) && (gw_regex_exec(t->keyword_regex, word, n_match, p_match, 0) == 0))
+            gwlist_append(list, t);
+
+        /* otherwise look for exact match */
+        if (octstr_compare(t->keyword, word) == 0) 
+            gwlist_append(list, t);
+	}
+    return list;
+}
+
 /*
  * Find the appropriate translation 
  */
 static URLTranslation *find_translation(URLTranslationList *trans, 
-	List *words, Octstr *smsc, Octstr *sender, Octstr *receiver, int *reject)
+                    List *words, Octstr *smsc, Octstr *sender, Octstr *receiver, int *reject)
 {
     Octstr *keyword;
     int i, n;
     URLTranslation *t;
     List *list;
-    
-    n = list_len(words);
+
+    n = gwlist_len(words);
     if (n == 0)
-	return NULL;
-    keyword = list_get(words, 0);
+        return NULL;
+    n = 1;
+
+    keyword = gwlist_get(words, 0);
     keyword = octstr_duplicate(keyword);
     octstr_convert_range(keyword, 0, octstr_len(keyword), tolower);
-    
-    list = dict_get(trans->dict, keyword);
+
+    list = get_matching_translations(trans, keyword);
+    /*
+      list now contains all translations where the keyword of the sms matches the
+      pattern defined by the tranlsation's keyword
+    */
     t = NULL;
-    for (i = 0; i < list_len(list); ++i) {
-	t = list_get(list, i);
+    for (i = 0; i < gwlist_len(list); ++i) {
+        t = gwlist_get(list, i);
 
-	/* if smsc_id set and accepted_smsc exist, accept
-	 * translation only if smsc id is in accept string
-	 */
-	if (smsc && t->accepted_smsc) {
-	    if (!list_search(t->accepted_smsc, smsc, octstr_item_match)) {
-		t = NULL;
-		continue;
-	    }
-	}
-
-	/* Have allowed for sender */
-	if (t->allowed_prefix && ! t->denied_prefix &&
-	   (does_prefix_match(t->allowed_prefix, sender) != 1)) {
-	    t = NULL;
-	    continue;
-	}
-
-	/* Have denied for sender */
-	if (t->denied_prefix && ! t->allowed_prefix &&
-	   (does_prefix_match(t->denied_prefix, sender) == 1)) {
-	    t = NULL;
-	    continue;
-	}
-
-	/* Have allowed for receiver */
-	if (t->allowed_recv_prefix && ! t->denied_recv_prefix &&
-	   (does_prefix_match(t->allowed_recv_prefix, receiver) != 1)) {
-	    t = NULL;
-	    continue;
-	}
-
-	/* Have denied for receiver */
-	if (t->denied_recv_prefix && ! t->allowed_recv_prefix &&
-	   (does_prefix_match(t->denied_recv_prefix, receiver) == 1)) {
-	    t = NULL;
-	    continue;
-	}
-
-	if (t->white_list &&
-	    numhash_find_number(t->white_list, sender) < 1) {
-	    t = NULL; *reject = 1;
-	    continue;
-	}   
-	if (t->black_list &&
-	    numhash_find_number(t->black_list, sender) == 1) {
-	    t = NULL; *reject = 1;
-	    continue;
-	}   
-
-	/* Have allowed and denied */
-	if (t->denied_prefix && t->allowed_prefix &&
-	   (does_prefix_match(t->allowed_prefix, sender) != 1) &&
-	   (does_prefix_match(t->denied_prefix, sender) == 1) ) {
-	    t = NULL;
-	    continue;
-	}
-
-	if (t->catch_all)
+        if (check_allowed_translation(t, smsc, sender, receiver, reject) == 0
+            && check_num_args(t, words) == 0)
 	    break;
 
-	if (n - 1 == t->args)
-	    break;
-	if (t->has_catchall_arg && n - 1 >= t->args)
-	    break;
 	t = NULL;
     }
 
@@ -1078,6 +1401,7 @@ static URLTranslation *find_translation(URLTranslationList *trans,
 	*reject = 0;
 
     octstr_destroy(keyword);    
+    gwlist_destroy(list, NULL);
     return t;
 }
 
@@ -1094,60 +1418,13 @@ static URLTranslation *find_default_translation(URLTranslationList *trans,
 
     list = dict_get(trans->dict, octstr_imm("default"));
     t = NULL;
-    for (i = 0; i < list_len(list); ++i) {
-	t = list_get(list, i);
-	if (smsc && t->accepted_smsc) {
-	    if (!list_search(t->accepted_smsc, smsc, octstr_item_match)) {
-		t = NULL;
-		continue;
-	    }
-	}
+    for (i = 0; i < gwlist_len(list); ++i) {
+	t = gwlist_get(list, i);
+
+    if (check_allowed_translation(t, smsc, sender, receiver, reject) == 0)
+        break;
 
-	/* Have allowed sender */
-	if (t->allowed_prefix && ! t->denied_prefix &&
-		       	(does_prefix_match(t->allowed_prefix, sender) != 1)) {
 	    t = NULL;
-	    continue;
-	}
-
-	/* Have denied sender */
-	if (t->denied_prefix && ! t->allowed_prefix &&
-		       	(does_prefix_match(t->denied_prefix, sender) == 1)) {
-	    t = NULL;
-	    continue;
-	}
-
-	/* Have allowed receiver */
-	if (t->allowed_recv_prefix && ! t->denied_recv_prefix &&
-		       	(does_prefix_match(t->allowed_recv_prefix, receiver) != 1)) {
-	    t = NULL;
-	    continue;
-	}
-
-	/* Have denied receiver */
-	if (t->denied_recv_prefix && ! t->allowed_recv_prefix &&
-		       	(does_prefix_match(t->denied_recv_prefix, receiver) == 1)) {
-	    t = NULL;
-	    continue;
-	}
-
-	if (t->white_list && numhash_find_number(t->white_list, sender) < 1) {
-	    t = NULL; *reject = 1;
-	    continue;
-	}
-	if (t->black_list && numhash_find_number(t->black_list, sender) == 1) {
-	    t = NULL; *reject = 1;
-	    continue;
-										        }
-
-	/* Have allowed and denied */
-	if (t->denied_prefix && t->allowed_prefix &&
-		       	(does_prefix_match(t->allowed_prefix, sender) != 1) &&
-		       	(does_prefix_match(t->denied_prefix, sender) == 1) ) {
-	    t = NULL;
-	    continue;
-	}
-	break;
     }
 
     /* Only return reject if there's only blacklisted smsc's */
@@ -1165,10 +1442,10 @@ static URLTranslation *find_black_list_translation(URLTranslationList *trans,
 
     list = dict_get(trans->dict, octstr_imm("black-list"));
     t = NULL;
-    for (i = 0; i < list_len(list); ++i) {
-	t = list_get(list, i);
+    for (i = 0; i < gwlist_len(list); ++i) {
+	t = gwlist_get(list, i);
 	if (smsc && t->accepted_smsc) {
-	    if (!list_search(t->accepted_smsc, smsc, octstr_item_match)) {
+	    if (!gwlist_search(t->accepted_smsc, smsc, octstr_item_match)) {
 		t = NULL;
 		continue;
 	    }
