@@ -1,3 +1,59 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2005 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*
  * wap_push_ppg_pushuser.c: Implementation of wap_push_ppg_pushuser.h header.
  *
@@ -6,12 +62,13 @@
 
 #include "wap_push_ppg_pushuser.h"
 #include "numhash.h"
+#include "gwlib/regex.h"
 
 /***************************************************************************
  *
  * Global data structures
  *
- * Hold authentication data for one ppg user
+ * Hold user specific  data for one ppg user
  */
 
 struct WAPPushUser {
@@ -21,19 +78,30 @@ struct WAPPushUser {
     Octstr *country_prefix;
     Octstr *allowed_prefix;            /* phone number prefixes allowed by 
                                           this user when pushing*/
+    regex_t *allowed_prefix_regex;
+    
     Octstr *denied_prefix;             /* and denied ones */
+    regex_t *denied_prefix_regex;
+
     Numhash *white_list;               /* phone numbers of this user, used for 
                                           push*/
+    regex_t *white_list_regex;
     Numhash *black_list;               /* numbers should not be used for push*/
+    regex_t *black_list_regex;
+
     Octstr *user_deny_ip;              /* this user allows pushes from these 
                                           IPs*/
     Octstr *user_allow_ip;             /* and denies them from these*/
+    Octstr *smsc_id;                   /* force push SMs to this smsc */
+    Octstr *default_smsc_id;           /* use this smsc as a default for push SMs */
+    Octstr *dlr_url;                   /* default dlr url from this user */
+    Octstr *smsbox_id;                 /* use this smsbox for sending dlrs back*/
 };
 
 typedef struct WAPPushUser WAPPushUser;
 
 /*
- * Hold authentication data of all ppg users
+ * Hold user specific  data of all ppg users
  */
 
 struct WAPPushUserList {
@@ -74,6 +142,8 @@ static void reply(HTTPClient *c, List *push_headers);
 static int parse_cgivars_for_username(List *cgivars, Octstr **username);
 static int parse_cgivars_for_password(List *cgivars, Octstr **password);
 static int compare_octstr_sequence(Octstr *os1, Octstr *os2, long start);
+static Octstr *forced_smsc(WAPPushUser *u);
+static Octstr *default_smsc(WAPPushUser *u);
 
 /****************************************************************************
  *
@@ -91,13 +161,13 @@ int wap_push_ppg_pushuser_list_add(List *list, long number_of_pushes,
     next_try = dict_create(number_of_pushes, octstr_destroy_item);
     users = pushusers_create(number_of_users);
     gw_assert(list);
-    while (list && (grp = list_extract_first(list))) {
+    while (list && (grp = gwlist_extract_first(list))) {
         if (oneuser_add(grp) == -1) {
-	        list_destroy(list, NULL);
+	        gwlist_destroy(list, NULL);
             return 0;
         }
     }
-    list_destroy(list, NULL);
+    gwlist_destroy(list, NULL);
 
     return 1;
 }
@@ -108,7 +178,7 @@ void wap_push_ppg_pushuser_list_destroy(void)
     if (users == NULL)
         return;
 
-    list_destroy(users->list, destroy_oneuser);
+    gwlist_destroy(users->list, destroy_oneuser);
     dict_destroy(users->names);
     gw_free(users);
 }
@@ -267,14 +337,14 @@ int wap_push_ppg_pushuser_client_phone_number_acceptable(Octstr *username,
 }
 
 int wap_push_ppg_pushuser_search_ip_from_wildcarded_list(Octstr *haystack, 
-        Octstr *needle, Octstr *list_sep, Octstr *ip_sep)
+        Octstr *needle, Octstr *gwlist_sep, Octstr *ip_sep)
 {
     List *ips;
     long i;
     Octstr *configured_ip;
 
     gw_assert(haystack);
-    gw_assert(list_sep);
+    gw_assert(gwlist_sep);
     gw_assert(ip_sep);
 
     /*There are no wildcards in the list*/    
@@ -288,21 +358,70 @@ int wap_push_ppg_pushuser_search_ip_from_wildcarded_list(Octstr *haystack,
     
     /*There are wildcards in the list*/
     configured_ip = NULL;
-    ips = octstr_split(haystack, list_sep);
-    for (i = 0; i < list_len(ips); ++i) {
-        configured_ip = list_get(ips, i);
+    ips = octstr_split(haystack, gwlist_sep);
+    for (i = 0; i < gwlist_len(ips); ++i) {
+        configured_ip = gwlist_get(ips, i);
         if (wildcarded_ip_found(configured_ip, needle, ip_sep))
 	        goto found;
     }
 
-    list_destroy(ips, octstr_destroy_item);
+    gwlist_destroy(ips, octstr_destroy_item);
     return 0;
 
 found:
-    list_destroy(ips, octstr_destroy_item);
+    gwlist_destroy(ips, octstr_destroy_item);
     return 1;
 }
 
+/*
+ * Returns smsc-id that pushes by this user must use, 
+ * NULL when there was an error.
+ */
+Octstr *wap_push_ppg_pushuser_smsc_id_get(Octstr *username)
+{
+    WAPPushUser *u;
+    Octstr *smsc_id;
+
+    if ((u = user_find_by_username(username)) == NULL) {
+        /* no user found with this username */
+        return NULL;
+    }
+
+    if ((smsc_id = forced_smsc(u)) != NULL)
+        return octstr_duplicate(smsc_id);
+
+    smsc_id = default_smsc(u);
+    return octstr_duplicate(smsc_id);
+}
+
+/*
+ * Returns default dlr url for this user.
+ */
+Octstr *wap_push_ppg_pushuser_dlr_url_get(Octstr *username)
+{
+    WAPPushUser *u;
+    Octstr *dlr_url;
+
+    u = user_find_by_username(username);
+    dlr_url = u->dlr_url;     
+
+    return octstr_duplicate(dlr_url);
+}
+
+/*
+ * Returns default dlr smsbox id for this user.
+ */
+Octstr *wap_push_ppg_pushuser_smsbox_id_get(Octstr *username)
+{
+    WAPPushUser *u;
+    Octstr *smsbox_id;
+
+    u = user_find_by_username(username);
+    smsbox_id = u->smsbox_id;
+
+    return octstr_duplicate(smsbox_id);
+}
+
 
 /***************************************************************************
  *
@@ -311,13 +430,13 @@ found:
 
 static void destroy_users_list(void *l)
 {
-    list_destroy(l, NULL);
+    gwlist_destroy(l, NULL);
 }
 
 static WAPPushUserList *pushusers_create(long number_of_users) 
 {
     users = gw_malloc(sizeof(WAPPushUserList));
-    users->list = list_create();
+    users->list = gwlist_create();
     users->names = dict_create(number_of_users, destroy_users_list);
 
     return users;
@@ -345,11 +464,17 @@ static WAPPushUser *create_oneuser(CfgGroup *grp)
     u->name = NULL;
     u->username = NULL;                  
     u->allowed_prefix = NULL;           
+    u->allowed_prefix_regex = NULL;           
     u->denied_prefix = NULL;             
+    u->denied_prefix_regex = NULL;             
     u->white_list = NULL;               
+    u->white_list_regex = NULL;               
     u->black_list = NULL;              
+    u->black_list_regex = NULL;              
     u->user_deny_ip = NULL;              
     u->user_allow_ip = NULL;
+    u->smsc_id = NULL;
+    u->default_smsc_id = NULL;
 
     u->name = cfg_get(grp, octstr_imm("wap-push-user"));
 
@@ -381,6 +506,10 @@ static WAPPushUser *create_oneuser(CfgGroup *grp)
     u->country_prefix = cfg_get(grp, octstr_imm("country-prefix"));
     u->allowed_prefix = cfg_get(grp, octstr_imm("allowed-prefix"));
     u->denied_prefix = cfg_get(grp, octstr_imm("denied-prefix"));
+    u->smsc_id = cfg_get(grp, octstr_imm("forced-smsc"));
+    u->default_smsc_id = cfg_get(grp, octstr_imm("default-smsc"));
+    u->dlr_url = cfg_get(grp, octstr_imm("dlr-url"));
+    u->smsbox_id = cfg_get(grp, octstr_imm("smsbox-id"));
 
     os = cfg_get(grp, octstr_imm("white-list"));
     if (os != NULL) {
@@ -393,6 +522,27 @@ static WAPPushUser *create_oneuser(CfgGroup *grp)
 	    octstr_destroy(os);
     }
 
+    if ((os = cfg_get(grp, octstr_imm("allowed-prefix-regex"))) != NULL) {
+        if ((u->allowed_prefix_regex = gw_regex_comp(os, REG_EXTENDED)) == NULL)
+            panic(0, "Could not compile pattern '%s'", octstr_get_cstr(os));
+        octstr_destroy(os);
+    };
+    if ((os = cfg_get(grp, octstr_imm("denied-prefix-regex"))) != NULL) {
+        if ((u->denied_prefix_regex = gw_regex_comp(os, REG_EXTENDED)) == NULL)
+            panic(0, "Could not compile pattern '%s'", octstr_get_cstr(os));
+        octstr_destroy(os);
+    };
+    if ((os = cfg_get(grp, octstr_imm("white-list-regex"))) != NULL) {
+        if ((u->white_list_regex = gw_regex_comp(os, REG_EXTENDED)) == NULL)
+            panic(0, "Could not compile pattern '%s'", octstr_get_cstr(os));
+        octstr_destroy(os);
+    };
+    if ((os = cfg_get(grp, octstr_imm("black-list-regex"))) != NULL) {
+        if ((u->black_list_regex = gw_regex_comp(os, REG_EXTENDED)) == NULL)
+            panic(0, "Could not compile pattern '%s'", octstr_get_cstr(os));
+        octstr_destroy(os);
+    };
+
     octstr_destroy(grpname);
     return u;
 
@@ -424,6 +574,13 @@ static void destroy_oneuser(void *p)
      numhash_destroy(u->black_list);              
      octstr_destroy(u->user_deny_ip);              
      octstr_destroy(u->user_allow_ip);
+     octstr_destroy(u->smsc_id);
+     octstr_destroy(u->default_smsc_id);
+
+     if (u->black_list_regex != NULL) gw_regex_destroy(u->black_list_regex);
+     if (u->white_list_regex != NULL) gw_regex_destroy(u->white_list_regex);
+     if (u->denied_prefix_regex != NULL) gw_regex_destroy(u->denied_prefix_regex);
+     if (u->allowed_prefix_regex != NULL) gw_regex_destroy(u->allowed_prefix_regex);
      gw_free(u);             
 }
 
@@ -449,6 +606,10 @@ static void oneuser_dump(WAPPushUser *u)
     octstr_dump(u->user_deny_ip, 0);    
     debug("wap.push.ppg.pushuser", 0, "allowed ip list:");                   
     octstr_dump(u->user_allow_ip, 0);
+    debug("wap.push.ppg.pushuser", 0, "send via smsc-id:");                   
+    octstr_dump(u->smsc_id, 0);
+    debug("wap.push.ppg.pushuser", 0, "use default smsc:");
+    octstr_dump(u->default_smsc_id, 0);
     debug("wap.push.ppg.pushuser", 0, "end of the dump");
 }
 
@@ -464,11 +625,11 @@ static int oneuser_add(CfgGroup *grp)
     if (u == NULL)
         return -1;
 
-    list_append(users->list, u);
+    gwlist_append(users->list, u);
 
     list = dict_get(users->names, u->username);
     if (list == NULL) {
-        list = list_create();
+        list = gwlist_create();
         dict_put(users->names, u->username, list);
     }
 
@@ -487,8 +648,8 @@ static WAPPushUser *user_find_by_username(Octstr *username)
     if ((list = dict_get(users->names, username)) == NULL)
          return NULL;
 
-    for (i = 0; i < list_len(users->list); ++i) {
-         u = list_get(users->list, i);
+    for (i = 0; i < gwlist_len(users->list); ++i) {
+         u = gwlist_get(users->list, i);
          if (octstr_compare(u->username, username) == 0)
 	         return u;
     }
@@ -515,22 +676,22 @@ static int wildcarded_ip_found(Octstr *ip, Octstr *needle, Octstr *ip_sep)
     ip_fragments = octstr_split(ip, ip_sep);
     needle_fragments = octstr_split(needle, ip_sep);
 
-    gw_assert(list_len(ip_fragments) == list_len(needle_fragments));
-    for (i = 0; i < list_len(ip_fragments); ++i) {
-        ip_fragment = list_get(ip_fragments, i);
-        needle_fragment = list_get(needle_fragments, i);
+    gw_assert(gwlist_len(ip_fragments) == gwlist_len(needle_fragments));
+    for (i = 0; i < gwlist_len(ip_fragments); ++i) {
+        ip_fragment = gwlist_get(ip_fragments, i);
+        needle_fragment = gwlist_get(needle_fragments, i);
         if (octstr_compare(ip_fragment, needle_fragment) != 0 && 
                 octstr_compare(ip_fragment, octstr_imm("*")) != 0)
  	        goto not_found;
     }
 
-    list_destroy(ip_fragments, octstr_destroy_item);
-    list_destroy(needle_fragments, octstr_destroy_item);   
+    gwlist_destroy(ip_fragments, octstr_destroy_item);
+    gwlist_destroy(needle_fragments, octstr_destroy_item);   
     return 1;
 
 not_found:
-    list_destroy(ip_fragments, octstr_destroy_item);
-    list_destroy(needle_fragments, octstr_destroy_item);
+    gwlist_destroy(ip_fragments, octstr_destroy_item);
+    gwlist_destroy(needle_fragments, octstr_destroy_item);
     return 0;
 }
 
@@ -631,11 +792,11 @@ static int response(List *push_headers, Octstr **username, Octstr **password)
     octstr_base64_to_binary(header_value);
     auth_list = octstr_split(header_value, octstr_imm(":"));
 
-    if (list_len(auth_list) != 2)
+    if (gwlist_len(auth_list) != 2)
         goto no_response2;
     
-    *username = octstr_duplicate(list_get(auth_list, 0));
-    *password = octstr_duplicate(list_get(auth_list, 1));
+    *username = octstr_duplicate(gwlist_get(auth_list, 0));
+    *password = octstr_duplicate(gwlist_get(auth_list, 1));
 
     if (username == NULL) {
         goto no_response2;
@@ -647,7 +808,7 @@ static int response(List *push_headers, Octstr **username, Octstr **password)
 
     debug("wap.push.ppg.pushuser", 0, "we have an username and a password in" 
           " authorization header");
-    list_destroy(auth_list, octstr_destroy_item);
+    gwlist_destroy(auth_list, octstr_destroy_item);
     octstr_destroy(header_value);
     return HEADER_AUTHENTICATION;
 
@@ -656,7 +817,7 @@ no_response1:
     return NO_USERNAME;
 
 no_response2:   
-    list_destroy(auth_list, octstr_destroy_item);
+    gwlist_destroy(auth_list, octstr_destroy_item);
     octstr_destroy(header_value);
     return NO_USERNAME;
 
@@ -664,7 +825,7 @@ no_response3:
     return NO_USERNAME;
 
 no_response4:   
-    list_destroy(auth_list, octstr_destroy_item);
+    gwlist_destroy(auth_list, octstr_destroy_item);
     octstr_destroy(header_value);
     return NO_PASSWORD;
 }
@@ -731,13 +892,14 @@ static int prefix_allowed(WAPPushUser *u, Octstr *number)
     if (u == NULL)
         goto no_user;
 
-    if (u->allowed_prefix == NULL && u->denied_prefix == NULL)
+    if (        u->allowed_prefix == NULL && u->denied_prefix == NULL 
+        && u->allowed_prefix_regex == NULL && u->denied_prefix_regex == NULL)
         goto no_configuration;
 
     if (u->denied_prefix != NULL) {
         denied = octstr_split(u->denied_prefix, octstr_imm(";"));
-        for (i = 0; i < list_len(denied); ++i) {
-             listed_prefix = list_get(denied, i);
+        for (i = 0; i < gwlist_len(denied); ++i) {
+             listed_prefix = gwlist_get(denied, i);
              if (u->country_prefix != NULL)
                  octstr_insert(listed_prefix, u->country_prefix, 0);
              if (compare_octstr_sequence(number, listed_prefix, 
@@ -747,13 +909,18 @@ static int prefix_allowed(WAPPushUser *u, Octstr *number)
         }
     }
 
-    if (u->allowed_prefix == NULL) {
-        goto no_allowed_config;
-    }
+    /* note: country-prefix _must_be included in the pattern */
+    if (u->denied_prefix_regex != NULL) 
+        if (gw_regex_match_pre(u->denied_prefix_regex, number) == 1)
+            goto denied;
 
+    if (u->allowed_prefix_regex == NULL && u->allowed_prefix == NULL) 
+        goto no_allowed_config;
+
+    if (u->allowed_prefix != NULL) {
     allowed = octstr_split(u->allowed_prefix, octstr_imm(";"));
-    for (i = 0; i < list_len(allowed); ++i) {
-         listed_prefix = list_get(allowed, i);
+    for (i = 0; i < gwlist_len(allowed); ++i) {
+         listed_prefix = gwlist_get(allowed, i);
          if (u->country_prefix != NULL)
              octstr_insert(listed_prefix, u->country_prefix, 0);
          if (compare_octstr_sequence(number, listed_prefix, 
@@ -761,19 +928,25 @@ static int prefix_allowed(WAPPushUser *u, Octstr *number)
 	         goto allowed;
          }
     }
+    }
+
+    /* note: country-prefix _must_ be included in the pattern */
+    if (u->allowed_prefix_regex != NULL) 
+        if (gw_regex_match_pre(u->allowed_prefix_regex, number) == 1)
+            goto allowed;
 
 /*
  * Here we have an intentional fall-through. It will removed when memory cleaning
  * functions are implemented.
  */
 denied:         
-    list_destroy(allowed, octstr_destroy_item);
-    list_destroy(denied, octstr_destroy_item);
+    gwlist_destroy(allowed, octstr_destroy_item);
+    gwlist_destroy(denied, octstr_destroy_item);
     return 0;
 
 allowed:      
-    list_destroy(allowed, octstr_destroy_item);
-    list_destroy(denied, octstr_destroy_item);
+    gwlist_destroy(allowed, octstr_destroy_item);
+    gwlist_destroy(denied, octstr_destroy_item);
     return 1;
 
 no_configuration:
@@ -783,24 +956,34 @@ no_user:
     return 0;
 
 no_allowed_config:
-    list_destroy(denied, octstr_destroy_item);
+    gwlist_destroy(denied, octstr_destroy_item);
     return 1;
 }
 
 static int whitelisted(WAPPushUser *u, Octstr *number)
 {
-    if (u->white_list == NULL)
-        return 1;
+    int result = 1;
 
-    return numhash_find_number(u->white_list, number);
+    if (u->white_list != NULL)
+        result = numhash_find_number(u->white_list, number);
+
+    if ((result == 0) && (u->white_list_regex != NULL))
+        result = gw_regex_match_pre(u->white_list_regex, number);
+
+    return result;
 }
 
 static int blacklisted(WAPPushUser *u, Octstr *number)
 {
-    if (u->black_list == NULL)
-        return 0;
+    int result = 0;
+    
+    if (u->black_list != NULL)
+        result = numhash_find_number(u->black_list, number);
 
-    return numhash_find_number(u->black_list, number);
+    if ((result == 0) && (u->black_list_regex != NULL))
+        result = gw_regex_match_pre(u->black_list_regex, number);
+
+    return result;
 }
 
 /* 
@@ -865,6 +1048,15 @@ static int compare_octstr_sequence(Octstr *os1, Octstr *os2, long start)
     return ret;
 }
 
+static Octstr *forced_smsc(WAPPushUser *u)
+{
+    return u->smsc_id;
+}
+
+static Octstr *default_smsc(WAPPushUser *u)
+{
+    return u->default_smsc_id;
+}
 
 
 
