@@ -1,3 +1,59 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2005 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /* conn.c - implement Connection type
  *
  * This file implements the interface defined in conn.h.
@@ -8,7 +64,7 @@
  * Jarkko Kovala <jarkko.kovala@iki.fi>
  *
  * SSL server implementation contributed by
- * Stipe Tolj <tolj@wapme-systems.de> for Wapme Systems AG
+ * Stipe Tolj <stolj@wapme.de> for Wapme Systems AG
  */
 
 /* TODO: unlocked_close() on error */
@@ -29,10 +85,8 @@
 #include <openssl/ssl.h>
 #include <openssl/err.h>
 
-SSL_CTX *global_ssl_context = NULL;
-SSL_CTX *global_server_ssl_context = NULL;
-X509 *ssl_public_cert;
-RSA *ssl_private_key;
+static SSL_CTX *global_ssl_context = NULL;
+static SSL_CTX *global_server_ssl_context = NULL;
 #endif /* HAVE_LIBSSL */
 
 typedef unsigned long (*CRYPTO_CALLBACK_PTR)(void);
@@ -62,6 +116,9 @@ struct Connection
     /* fd value is read-only and is not locked */
     int fd;
 
+    /* socket state */
+    enum {yes,no} connected;
+
     /* Protected by outlock */
     Octstr *outbuf;
     long outbufpos;   /* start of unwritten data in outbuf */
@@ -75,13 +132,14 @@ struct Connection
     long inbufpos;    /* start of unread data in inbuf */
 
     int read_eof;     /* we encountered eof on read */
-    int read_error;   /* we encountered error on read */
+    int io_error;   /* we encountered error on IO operation */
 
     /* Protected by both locks when updating, so you need only one
      * of the locks when reading. */
     FDSet *registered;
     conn_callback_t *callback;
     void *callback_data;
+    conn_callback_data_destroyer_t *callback_data_destroyer;
     /* Protected by inlock */
     int listening_pollin;
     /* Protected by outlock */
@@ -90,7 +148,6 @@ struct Connection
 #ifdef HAVE_LIBSSL
     SSL *ssl;
     X509 *peer_certificate;
-    Mutex *ssl_mutex;
 #endif /* HAVE_LIBSSL */
 };
 
@@ -110,7 +167,7 @@ static void unlocked_register_pollout(Connection *conn, int onoff);
 #define unlock_out(conn) unlock_out_real(conn, __FILE__, __LINE__, __func__)
 
 /* Lock a Connection's read direction, if the Connection is unclaimed */
-static void lock_in(Connection *conn)
+static void inline lock_in(Connection *conn)
 {
     gw_assert(conn != NULL);
 
@@ -121,23 +178,21 @@ static void lock_in(Connection *conn)
 }
 
 /* Unlock a Connection's read direction, if the Connection is unclaimed */
-static void unlock_in_real(Connection *conn, char *file, int line, const char *func)
+static void inline unlock_in_real(Connection *conn, char *file, int line, const char *func)
 {
     int ret;
     gw_assert(conn != NULL);
 
-    if (!conn->claimed) {
-        if ((ret = mutex_unlock(conn->inlock)) != 0) {
-            panic(0, "%s:%ld: %s: Mutex unlock failed. " \
-		             "(Called from %s:%ld:%s.)", \
-			         __FILE__, (long) __LINE__, __func__, \
-			         file, (long) line, func);
-        }
-     }
+    if (!conn->claimed && (ret = mutex_unlock(conn->inlock)) != 0) {
+        panic(0, "%s:%ld: %s: Mutex unlock failed. "
+            "(Called from %s:%ld:%s.)",
+            __FILE__, (long) __LINE__, __func__,
+            file, (long) line, func);
+    }
 }
 
 /* Lock a Connection's write direction, if the Connection is unclaimed */
-static void lock_out(Connection *conn)
+static void inline lock_out(Connection *conn)
 {
     gw_assert(conn != NULL);
 
@@ -148,29 +203,27 @@ static void lock_out(Connection *conn)
 }
 
 /* Unlock a Connection's write direction, if the Connection is unclaimed */
-static void unlock_out_real(Connection *conn, char *file, int line, const char *func)
+static void inline unlock_out_real(Connection *conn, char *file, int line, const char *func)
 {
     int ret;
     gw_assert(conn != NULL);
 
-    if (!conn->claimed) {
-        if ((ret = mutex_unlock(conn->outlock)) != 0) {
-            panic(0, "%s:%ld: %s: Mutex unlock failed. " \
-		             "(Called from %s:%ld:%s.)", \
-			         __FILE__, (long) __LINE__, __func__, \
-			         file, (long) line, func);
-        }
-     }
+    if (!conn->claimed && (ret = mutex_unlock(conn->outlock)) != 0) {
+        panic(0, "%s:%ld: %s: Mutex unlock failed. "
+            "(Called from %s:%ld:%s.)",
+            __FILE__, (long) __LINE__, __func__,
+            file, (long) line, func);
+    }
 }
 
 /* Return the number of bytes in the Connection's output buffer */
-static long unlocked_outbuf_len(Connection *conn)
+static long inline unlocked_outbuf_len(Connection *conn)
 {
     return octstr_len(conn->outbuf) - conn->outbufpos;
 }
 
 /* Return the number of bytes in the Connection's input buffer */
-static long unlocked_inbuf_len(Connection *conn)
+static long inline unlocked_inbuf_len(Connection *conn)
 {
     return octstr_len(conn->inbuf) - conn->inbufpos;
 }
@@ -179,39 +232,34 @@ static long unlocked_inbuf_len(Connection *conn)
  * of bytes written, or -1 in case of error. */
 static long unlocked_write(Connection *conn)
 {
-    long ret;
+    long ret = 0;
 
 #ifdef HAVE_LIBSSL
     if (conn->ssl != NULL) {
-        mutex_lock(conn->ssl_mutex);
-        ret = SSL_write(conn->ssl, 
-                        octstr_get_cstr(conn->outbuf) + conn->outbufpos, 
+        if (octstr_len(conn->outbuf) - conn->outbufpos > 0)
+            ret = SSL_write(conn->ssl,
+                        octstr_get_cstr(conn->outbuf) + conn->outbufpos,
                         octstr_len(conn->outbuf) - conn->outbufpos);
 
         if (ret < 0) {
-            int SSL_error = SSL_get_error(conn->ssl, ret); 
+            int SSL_error = SSL_get_error(conn->ssl, ret);
 
-            if (SSL_error == SSL_ERROR_WANT_READ) {
-                SSL_read(conn->ssl, NULL, 0);
-                mutex_unlock(conn->ssl_mutex);
-                return 0;
-            } else if (SSL_error == SSL_ERROR_WANT_WRITE) {
-                mutex_unlock(conn->ssl_mutex);
-                return 0;
+            if (SSL_error == SSL_ERROR_WANT_READ || SSL_error == SSL_ERROR_WANT_WRITE) {
+                  ret = 0; /* no error */
             } else {
-                error(0, "SSL write failed: OpenSSL error %d: %s", 
+                error(errno, "SSL write failed: OpenSSL error %d: %s",
                       SSL_error, ERR_error_string(SSL_error, NULL));
-                mutex_unlock(conn->ssl_mutex);
                 return -1;
             }
         }
-        mutex_unlock(conn->ssl_mutex);
     } else
 #endif /* HAVE_LIBSSL */
         ret = octstr_write_data(conn->outbuf, conn->fd, conn->outbufpos);
 
-    if (ret < 0)
+    if (ret < 0) {
+        conn->io_error = 1;
         return -1;
+    }
 
     conn->outbufpos += ret;
 
@@ -264,24 +312,7 @@ static void unlocked_read(Connection *conn)
 
 #ifdef HAVE_LIBSSL
     if (conn->ssl != NULL) {
-        mutex_lock(conn->ssl_mutex);
         len = SSL_read(conn->ssl, buf, sizeof(buf));
-
-        if (len < 0) { 
-            int SSL_error = SSL_get_error(conn->ssl, len);
-
-            if (SSL_error == SSL_ERROR_WANT_WRITE) {
-                len = SSL_write(conn->ssl, NULL, 0);
-                mutex_unlock(conn->ssl_mutex);
-                return;
-            } else if (SSL_error == SSL_ERROR_WANT_READ) {
-                mutex_unlock(conn->ssl_mutex);
-                return;
-            } else
-                error(0, "SSL read failed: OpenSSL error %d: %s",
-                      SSL_error, ERR_error_string(SSL_error, NULL));
-        }
-        mutex_unlock(conn->ssl_mutex);
     } else
 #endif /* HAVE_LIBSSL */
         len = read(conn->fd, buf, sizeof(buf));
@@ -289,8 +320,18 @@ static void unlocked_read(Connection *conn)
     if (len < 0) {
         if (errno == EINTR || errno == EAGAIN || errno == EWOULDBLOCK)
             return;
-        error(errno, "Error reading from fd %d:", conn->fd);
-        conn->read_error = 1;
+#ifdef HAVE_LIBSSL
+        if (conn->ssl) {
+            int SSL_error = SSL_get_error(conn->ssl, len);
+            if (SSL_error == SSL_ERROR_WANT_WRITE || SSL_error == SSL_ERROR_WANT_READ)
+                return; /* no error */
+            error(errno, "SSL read failed: OpenSSL error %d: %s",
+                      SSL_error, ERR_error_string(SSL_error, NULL));
+        }
+        else
+#endif /* HAVE_LIBSSL */
+            error(errno, "Error reading from fd %d:", conn->fd);
+        conn->io_error = 1;
         if (conn->registered)
             unlocked_register_pollin(conn, 0);
         return;
@@ -361,9 +402,6 @@ Connection *conn_open_ssl(Octstr *host, int port, Octstr *certkeyfile,
 			  Octstr *our_host)
 {
     Connection *ret;
-    int SSL_ret = 0;
-    int connected = 0;
-    time_t timeout;
 
     /* open the TCP connection */
     if (!(ret = conn_open_tcp(host, port, our_host))) {
@@ -371,23 +409,6 @@ Connection *conn_open_ssl(Octstr *host, int port, Octstr *certkeyfile,
     }
 
     ret->ssl = SSL_new(global_ssl_context);
-    ret->ssl_mutex = mutex_create();
-    
-    SSL_set_connect_state(ret->ssl);
-    if (certkeyfile != NULL) {
-        SSL_use_certificate_file(ret->ssl, octstr_get_cstr(certkeyfile),
-                                 SSL_FILETYPE_PEM);
-        SSL_use_PrivateKey_file(ret->ssl, octstr_get_cstr(certkeyfile), 
-                                SSL_FILETYPE_PEM);
-        if (SSL_check_private_key(ret->ssl) != 1) {
-            error(0, "conn_open_ssl: private key isn't consistent with the "
-                     "certificate from file %s (or failed reading the file)",
-                  octstr_get_cstr(certkeyfile));
-            goto error;
-        }
-    }
-    
-    SSL_set_fd(ret->ssl, ret->fd);
 
     /*
      * The current thread's error queue must be empty before
@@ -396,58 +417,39 @@ Connection *conn_open_ssl(Octstr *host, int port, Octstr *certkeyfile,
      */
     ERR_clear_error();
 
-    /*
-     * make the socket is non-blocking while we do SSL_connect
-     */
-    if (socket_set_blocking(ret->fd, 0) < 0) {
-        goto error;
-    }
-
-    /* record current time */
-    timeout = time(NULL);
-
-    while(!connected && (timeout + SSL_CONN_TIMEOUT > time(NULL))) {
-        /* Attempt to connect as long as the timeout hasn't run down */
-        SSL_ret = SSL_connect(ret->ssl);
-        switch(SSL_get_error(ret->ssl,SSL_ret)) {
-            case SSL_ERROR_WANT_READ:
-            case SSL_ERROR_WANT_WRITE:
-                /* non-blocking socket wants more time to read or write */
-                gwthread_sleep(0.01F);
-                continue;
-            default:
-                /* we're connected to the server successfuly */
-                connected++;
+    if (certkeyfile != NULL) {
+        SSL_use_certificate_file(ret->ssl, octstr_get_cstr(certkeyfile),
+                                 SSL_FILETYPE_PEM);
+        SSL_use_PrivateKey_file(ret->ssl, octstr_get_cstr(certkeyfile),
+                                SSL_FILETYPE_PEM);
+        if (SSL_check_private_key(ret->ssl) != 1) {
+            error(0, "conn_open_ssl: private key isn't consistent with the "
+                     "certificate from file %s (or failed reading the file)",
+                  octstr_get_cstr(certkeyfile));
+            goto error;
         }
     }
 
-    if (!connected) {
-        /* connection timed out - this probably means that something is terrible wrong */
-        int SSL_error = SSL_get_error (ret->ssl, SSL_ret);
-        error(0,"SSL connection timeout: OpenSSL error %d: %s",
-                SSL_error, ERR_error_string(SSL_error, NULL));
+    /* SSL_set_fd can fail, so check it */
+    if (SSL_set_fd(ret->ssl, ret->fd) == 0) {
+        /* SSL_set_fd failed, log error */
+        error(errno, "SSL: OpenSSL: %.256s", ERR_error_string(ERR_get_error(), NULL));
         goto error;
     }
-    
-    /* 
-     * XXX - restore the non-blocking state
-     * we don't need this since we use non-blocking operations
-     * anyway before doing SSL_connect(), right?!
-     */
+
     /*
+     * make sure the socket is non-blocking while we do SSL_connect
+     */
     if (socket_set_blocking(ret->fd, 0) < 0) {
         goto error;
     }
-    */
+    BIO_set_nbio(SSL_get_rbio(ret->ssl), 1);
+    BIO_set_nbio(SSL_get_wbio(ret->ssl), 1);
+
+    SSL_set_connect_state(ret->ssl);
 
-    if (SSL_ret != 1) {
-        int SSL_error = SSL_get_error (ret->ssl, SSL_ret); 
-        error(0, "SSL connect failed: OpenSSL error %d: %s", 
-                  SSL_error, ERR_error_string(SSL_error, NULL));
-        goto error;
-    }
-    
     return ret;
+
 error:
     conn_destroy(ret);
     return NULL;
@@ -460,6 +462,52 @@ Connection *conn_open_tcp(Octstr *host, int port, Octstr *our_host)
     return conn_open_tcp_with_port(host, port, 0, our_host);
 }
 
+Connection *conn_open_tcp_nb(Octstr *host, int port, Octstr *our_host)
+{
+  return conn_open_tcp_nb_with_port(host, port, 0, our_host);
+}
+
+Connection *conn_open_tcp_nb_with_port(Octstr *host, int port, int our_port,
+				       Octstr *our_host)
+{
+  int sockfd;
+  int done = -1;
+  Connection *c;
+
+  sockfd = tcpip_connect_nb_to_server_with_port(octstr_get_cstr(host), port,
+						our_port, our_host == NULL ?
+						NULL : octstr_get_cstr(our_host), &done);
+  if (sockfd < 0)
+    return NULL;
+  c = conn_wrap_fd(sockfd, 0);
+  if (done != 0) {
+    c->connected = no;
+  }
+  return c;
+}
+
+int conn_is_connected(Connection *conn)
+{
+  return conn->connected == yes ? 0 : -1;
+}
+
+int conn_get_connect_result(Connection *conn)
+{
+  int err;
+  socklen_t len;
+
+  len = sizeof(err);
+  if (getsockopt(conn->fd, SOL_SOCKET, SO_ERROR, &err, &len) < 0) {
+    return -1;
+  }
+  
+  if (err) {
+    return -1;
+  }
+  
+  conn->connected = yes;
+  return 0;
+}
 
 Connection *conn_open_tcp_with_port(Octstr *host, int port, int our_port,
 		Octstr *our_host)
@@ -493,13 +541,15 @@ Connection *conn_wrap_fd(int fd, int ssl)
     conn->inbufpos = 0;
 
     conn->fd = fd;
+    conn->connected = yes;
     conn->read_eof = 0;
-    conn->read_error = 0;
+    conn->io_error = 0;
     conn->output_buffering = DEFAULT_OUTPUT_BUFFERING;
 
     conn->registered = NULL;
     conn->callback = NULL;
     conn->callback_data = NULL;
+    conn->callback_data_destroyer = NULL;
     conn->listening_pollin = 0;
     conn->listening_pollout = 0;
 #ifdef HAVE_LIBSSL
@@ -507,111 +557,27 @@ Connection *conn_wrap_fd(int fd, int ssl)
      * do all the SSL magic for this connection
      */
     if (ssl) {
-        int rc;
-
         conn->ssl = SSL_new(global_server_ssl_context);
         conn->peer_certificate = NULL;
 
-        SSL_set_fd(conn->ssl, conn->fd);
+        /* SSL_set_fd can fail, so check it */
+        if (SSL_set_fd(conn->ssl, conn->fd) == 0) {
+            /* SSL_set_fd failed, log error and return NULL */
+            error(errno, "SSL: OpenSSL: %.256s", ERR_error_string(ERR_get_error(), NULL));
+            conn_destroy(conn);
+            return NULL;
+        }
         /* SSL_set_verify(conn->ssl, 0, NULL); */
-        BIO_set_nbio(SSL_get_rbio(conn->ssl), 0);
-        BIO_set_nbio(SSL_get_wbio(conn->ssl), 0);
 
-        conn->ssl_mutex = mutex_create();
+        /* set read/write BIO layer to non-blocking mode */
+        BIO_set_nbio(SSL_get_rbio(conn->ssl), 1);
+        BIO_set_nbio(SSL_get_wbio(conn->ssl), 1);
 
-        /* 
-         * now enter the SSL handshake phase
-         */    
-     
-        /*
-         * For non-blocking BIO we may return from SSL_accept(). In this 
-         * case we check for SSL_get_error() = SSL_ERROR_WANT_[READ|WRITE]
-         * and loop the SSL_accept() until we have come through.
-         */
-        while (((rc = SSL_accept(conn->ssl)) <= 0) && 
-               ((SSL_get_error(conn->ssl, rc) == SSL_ERROR_WANT_READ) ||
-               (SSL_get_error(conn->ssl, rc) == SSL_ERROR_WANT_WRITE))) 
-            {}
-	  	     
-        /*
-         * If SSL_accept() has failed then check which reason it may 
-         * have been and log the error.
-         */
-        if (rc <= 0) {
-             
-            if (SSL_get_error(conn->ssl, rc) == SSL_ERROR_ZERO_RETURN) {
-                /*
-                 * The case where the connection was closed before any data
-                 * was transferred. That's not a real error and can occur
-                 * sporadically with some clients.
-                 */
-                warning(0, "SSL: handshake stopped: connection was closed");
-                warning(0, "SSL: OpenSSL: %.256s", ERR_error_string(ERR_get_error(), NULL));
-
-                SSL_set_shutdown(conn->ssl, SSL_RECEIVED_SHUTDOWN);
-                SSL_smart_shutdown(conn->ssl);
-             }
-             else if (ERR_GET_REASON(ERR_peek_error()) == SSL_R_HTTP_REQUEST) {
-                /*
-                 * The case where OpenSSL has recognized a HTTP request:
-                 * This means the client speaks plain HTTP on our HTTPS
-                 * port. Hmmmm...  At least for this error we can be more friendly
-                 * and try to provide him with a HTML error page. We have only one
-                 * problem: OpenSSL has already read some bytes from the HTTP
-                 * request. So we have to skip the request line manually.
-                 */
-                char ca[2];
-                int rv;
-
-                warning(0, "SSL: handshake failed: HTTP spoken on HTTPS port");
-                warning(0, "SSL: OpenSSL: %.256s", ERR_error_string(ERR_get_error(), NULL));
-                    
-                /* first: skip the remaining bytes of the request line */
-                do {
-                    do {
-                        rv = read(conn->fd, ca, 1);
-                    } while (rv == -1 && errno == EINTR);
-                } while (rv > 0 && ca[0] != '\012' /*LF*/);
-
-                /* second: kick away the SSL stuff */
-                SSL_set_shutdown(conn->ssl, SSL_SENT_SHUTDOWN|SSL_RECEIVED_SHUTDOWN);
-                SSL_smart_shutdown(conn->ssl);
-
-                /* tell the user how to access using the HTTPS scheme */
-                //SSL_http_hint(conn, HTTP_BAD_REQUEST);
-             }   
-             else if (SSL_get_error(conn->ssl, rc) == SSL_ERROR_SYSCALL) {
-                if (errno > 0)
-                    warning(0, "SSL: handshake interrupted by system (stop button pressed in browser?!)");
-                else
-                    warning(0, "SSL: spurious handshake interrupt (one of OpenSSL confusions?!)");
-                error(0, "SSL: OpenSSL: %.256s", ERR_error_string(ERR_get_error(), NULL));
-
-                SSL_set_shutdown(conn->ssl, SSL_RECEIVED_SHUTDOWN);
-                SSL_smart_shutdown(conn->ssl);
-             } 
-             else {
-                /*
-                 * ok, anything else is a fatal error
-                 */
-                warning(0, "SSL: handshake failed with fatal error");
-                warning(0, "SSL: OpenSSL: %.256s", ERR_error_string(ERR_get_error(), NULL));
-                
-                SSL_set_shutdown(conn->ssl, SSL_RECEIVED_SHUTDOWN);
-                SSL_smart_shutdown(conn->ssl);
-             }
-
-             warning(0, "SSL: disconnecting.");
-  
-             conn_destroy(conn);
-             return NULL;
-  
-        } /* SSL error */
-     
+        /* set accept state , SSL-Handshake will be handled transparent while SSL_[read|write] */
+         SSL_set_accept_state(conn->ssl);
     } else {
         conn->ssl = NULL;
         conn->peer_certificate = NULL;
-        conn->ssl_mutex = NULL;
     }
 #endif /* HAVE_LIBSSL */
 
@@ -628,27 +594,25 @@ void conn_destroy(Connection *conn)
     /* No locking done here.  conn_destroy should not be called
      * if any thread might still be interested in the connection. */
 
-    if (conn->registered)
+    if (conn->registered) {
         fdset_unregister(conn->registered, conn->fd);
+        /* call data destroyer if any */
+        if (conn->callback_data != NULL && conn->callback_data_destroyer != NULL)
+            conn->callback_data_destroyer(conn->callback_data);
+    }
 
     if (conn->fd >= 0) {
-        /* Try to flush any remaining data, only in case this is
-         * not a SSL connection. Otherwise this crashes the bearerbox
-         * at least on the Cygwin platform
-         */
+        /* Try to flush any remaining data */
+        unlocked_try_write(conn);
+
 #ifdef HAVE_LIBSSL
         if (conn->ssl != NULL) {
-            mutex_lock(conn->ssl_mutex);
-            SSL_shutdown(conn->ssl);
+            SSL_smart_shutdown(conn->ssl);
             SSL_free(conn->ssl);
-            if (conn->peer_certificate != NULL) 
+            if (conn->peer_certificate != NULL)
                 X509_free(conn->peer_certificate);
-            mutex_unlock(conn->ssl_mutex);
-            mutex_destroy(conn->ssl_mutex);
-	    }
-        else
+	}
 #endif /* HAVE_LIBSSL */
-        unlocked_write(conn);
 
         ret = close(conn->fd);
         if (ret < 0)
@@ -709,13 +673,15 @@ int conn_eof(Connection *conn)
     return eof;
 }
 
-int conn_read_error(Connection *conn)
+int conn_error(Connection *conn)
 {
     int err;
 
+    lock_out(conn);
     lock_in(conn);
-    err = conn->read_error;
+    err = conn->io_error;
     unlock_in(conn);
+    unlock_out(conn);
 
     return err;
 }
@@ -746,30 +712,55 @@ static void poll_callback(int fd, int revents, void *data)
         return;
     }
 
+    /* Get result of nonblocking connect, before any reads and writes
+     * we must check result (it must be handled in initial callback) */
+    if (conn->connected == no) {
+        if (conn->callback)
+            conn->callback(conn, conn->callback_data);
+        return;
+    }
+
+    /* If got POLLERR or POLHUP, then unregister the descriptor from the
+     * fdset and set the error condition variable to let the upper layer
+     * close and destroy the connection. */
+    if (revents & (POLLERR|POLLHUP)) {
+        lock_out(conn);
+        lock_in(conn);
+        if (conn->listening_pollin)
+            unlocked_register_pollin(conn, 0);
+        if (conn->listening_pollout)
+            unlocked_register_pollout(conn, 0);
+        conn->io_error = 1;
+        unlock_in(conn);
+        unlock_out(conn);
+        do_callback = 1;
+    }
+
     /* If unlocked_write manages to write all pending data, it will
      * tell the fdset to stop listening for POLLOUT. */
     if (revents & POLLOUT) {
         lock_out(conn);
         unlocked_write(conn);
-	if (unlocked_outbuf_len(conn) == 0)
-	    do_callback = 1;
+        if (unlocked_outbuf_len(conn) == 0)
+            do_callback = 1;
         unlock_out(conn);
     }
 
-    /* If unlocked_read hits eof or error, it will tell the fdset to
-     * stop listening for POLLIN. */
-    if (revents & (POLLIN | POLLERR)) {
+    /* We read only in unlocked_read in we received POLLIN, cause the 
+     * descriptor is already broken and of no use anymore. */
+    if (revents & POLLIN) {
         lock_in(conn);
         unlocked_read(conn);
         unlock_in(conn);
-	do_callback = 1;
+        do_callback = 1;
     }
-    if (do_callback)
+
+    if (do_callback && conn->callback)
         conn->callback(conn, conn->callback_data);
 }
 
-int conn_register(Connection *conn, FDSet *fdset,
-                  conn_callback_t callback, void *data)
+int conn_register_real(Connection *conn, FDSet *fdset,
+                  conn_callback_t callback, void *data, conn_callback_data_destroyer_t *data_destroyer)
 {
     int events;
     int result = 0;
@@ -787,38 +778,51 @@ int conn_register(Connection *conn, FDSet *fdset,
     if (conn->registered == fdset) {
         /* Re-registering.  Change only the callback info. */
         conn->callback = callback;
+        /* call data destroyer if new data supplied */
+        if (conn->callback_data != NULL && conn->callback_data != data && conn->callback_data_destroyer != NULL)
+            conn->callback_data_destroyer(conn->callback_data);
         conn->callback_data = data;
+        conn->callback_data_destroyer = data_destroyer;
         result = 0;
     } else if (conn->registered) {
         /* Already registered to a different fdset. */
         result = -1;
     } else {
         events = 0;
-        if (conn->read_eof == 0 && conn->read_error == 0)
-            events |= POLLIN;
-        if (unlocked_outbuf_len(conn) > 0)
-            events |= POLLOUT;
+	/* For nonconnected socket we must lesten both directions */
+        if (conn->connected == yes) {
+            if (conn->read_eof == 0 && conn->io_error == 0)
+                events |= POLLIN;
+            if (unlocked_outbuf_len(conn) > 0)
+                events |= POLLOUT;
+        } else {
+          events |= POLLIN | POLLOUT;
+        }
 
         conn->registered = fdset;
         conn->callback = callback;
         conn->callback_data = data;
+        conn->callback_data_destroyer = data_destroyer;
         conn->listening_pollin = (events & POLLIN) != 0;
         conn->listening_pollout = (events & POLLOUT) != 0;
         fdset_register(fdset, conn->fd, events, poll_callback, conn);
         result = 0;
     }
 
-    unlock_out(conn);
     unlock_in(conn);
+    unlock_out(conn);
 
     return result;
 }
 
 void conn_unregister(Connection *conn)
 {
+    FDSet *set = NULL;
+    int fd;
+    
     gw_assert(conn != NULL);
 
-    if (conn->fd < 0)
+    if (conn == NULL || conn->fd < 0)
         return;
 
     /* We need both locks to update the registration information */
@@ -826,16 +830,25 @@ void conn_unregister(Connection *conn)
     lock_in(conn);
 
     if (conn->registered) {
-        fdset_unregister(conn->registered, conn->fd);
+        set = conn->registered;
+        fd = conn->fd;
         conn->registered = NULL;
         conn->callback = NULL;
+        /* call data destroyer */
+        if (conn->callback_data != NULL && conn->callback_data_destroyer != NULL)
+            conn->callback_data_destroyer(conn->callback_data);
         conn->callback_data = NULL;
+        conn->callback_data_destroyer = NULL;
         conn->listening_pollin = 0;
         conn->listening_pollout = 0;
     }
 
     unlock_in(conn);
     unlock_out(conn);
+    
+    /* now unregister from FDSet */
+    if (set != NULL)
+        fdset_unregister(set, fd);
 }
 
 int conn_wait(Connection *conn, double seconds)
@@ -877,7 +890,7 @@ int conn_wait(Connection *conn, double seconds)
 
     /* We need the in lock to query read_eof */
     lock_in(conn);
-    if ((conn->read_eof == 0 && conn->read_error == 0) || events == 0)
+    if ((conn->read_eof == 0 && conn->io_error == 0) || events == 0)
         events |= POLLIN;
     unlock_in(conn);
 
@@ -1048,6 +1061,9 @@ Octstr *conn_read_fixed(Connection *conn, long length)
 {
     Octstr *result = NULL;
 
+    if (length < 1)
+        return NULL;
+
     /* See if the data is already available.  If not, try a read(),
      * then see if we have enough data after that.  If not, give up. */
     lock_in(conn);
@@ -1104,8 +1120,8 @@ Octstr *conn_read_withlen(Connection *conn)
 {
     Octstr *result = NULL;
     unsigned char lengthbuf[4];
-    long length;
-    int try;
+    long length = 0; /* for compiler please */
+    int try, retry;
 
     lock_in(conn);
 
@@ -1113,19 +1129,21 @@ Octstr *conn_read_withlen(Connection *conn)
         if (try > 1)
             unlocked_read(conn);
 
-retry:
-        /* First get the length. */
-        if (unlocked_inbuf_len(conn) < 4)
-            continue;
+        do {
+            retry = 0;
+            /* First get the length. */
+            if (unlocked_inbuf_len(conn) < 4)
+                continue;
 
-        octstr_get_many_chars(lengthbuf, conn->inbuf, conn->inbufpos, 4);
-        length = decode_network_long(lengthbuf);
+            octstr_get_many_chars(lengthbuf, conn->inbuf, conn->inbufpos, 4);
+            length = decode_network_long(lengthbuf);
 
-        if (length < 0) {
-            warning(0, "conn_read_withlen: got negative length, skipping");
-            conn->inbufpos += 4;
-            goto retry;
-         }
+            if (length < 0) {
+                warning(0, "conn_read_withlen: got negative length, skipping");
+                conn->inbufpos += 4;
+                retry = 1;
+             }
+        } while(retry == 1);
 
         /* Then get the data. */
         if (unlocked_inbuf_len(conn) - 4 < length)
@@ -1179,16 +1197,26 @@ Octstr *conn_read_packet(Connection *conn, int startmark, int endmark)
 #ifdef HAVE_LIBSSL
 X509 *conn_get_peer_certificate(Connection *conn) 
 {
-    mutex_lock(conn->ssl_mutex);
-    if (conn->peer_certificate == NULL && conn->ssl != NULL) 
+    /* Don't know if it needed to be locked , but better safe as crash */
+    lock_out(conn);
+    lock_in(conn);
+    if (conn->peer_certificate == NULL && conn->ssl != NULL)
         conn->peer_certificate = SSL_get_peer_certificate(conn->ssl);
-    mutex_unlock(conn->ssl_mutex);
-    return(conn->peer_certificate);
+    unlock_in(conn);
+    unlock_out(conn);
+    
+    return conn->peer_certificate;
 }
 
-RSA *tmp_rsa_callback(SSL *ssl, int export, int key_len) 
+/*
+ * XXX Alex decalred the RSA callback routine static and now we're getting 
+ * warning messages for our automatic compilation tests. So we are commenting
+ * the function out to avoid the warnings.
+ *
+
+static RSA *tmp_rsa_callback(SSL *ssl, int export, int key_len)
 {
-    static RSA *rsa = NULL; 
+    static RSA *rsa = NULL;
     debug("gwlib.http", 0, "SSL: Generating new RSA key (export=%d, keylen=%d)", export, key_len);
     if (export) {
 	   rsa = RSA_generate_key(key_len, RSA_F4, NULL, NULL);
@@ -1197,11 +1225,12 @@ RSA *tmp_rsa_callback(SSL *ssl, int export, int key_len)
     }
     return rsa;
 }
+*/
 
-Mutex **ssl_static_locks = NULL;
-Mutex **ssl_server_static_locks = NULL;
+static Mutex **ssl_static_locks = NULL;
 
-void openssl_locking_function(int mode, int n, const char *file, int line) 
+/* the call-back function for the openssl crypto thread locking */
+static void openssl_locking_function(int mode, int n, const char *file, int line)
 {
     if (mode & CRYPTO_LOCK)
         mutex_lock(ssl_static_locks[n-1]);
@@ -1209,46 +1238,53 @@ void openssl_locking_function(int mode, int n, const char *file, int line)
         mutex_unlock(ssl_static_locks[n-1]);
 }
 
-void openssl_server_locking_function(int mode, int n, const char *file, int line) 
-{
-    if (mode & CRYPTO_LOCK)
-        mutex_lock(ssl_server_static_locks[n-1]);
-    else
-        mutex_unlock(ssl_server_static_locks[n-1]);
-}
-
-void conn_init_ssl(void)
+void openssl_init_locks(void)
 {
     int c, maxlocks = CRYPTO_num_locks();
 
     gw_assert(ssl_static_locks == NULL);
+
     ssl_static_locks = gw_malloc(sizeof(Mutex *) * maxlocks);
-    for (c=0;c<maxlocks;c++) 
+    for (c = 0; c < maxlocks; c++)
         ssl_static_locks[c] = mutex_create();
 
+    /* after the mutexes have been created, apply the call-back to it */
     CRYPTO_set_locking_callback(openssl_locking_function);
     CRYPTO_set_id_callback((CRYPTO_CALLBACK_PTR)gwthread_self);
+}
 
+void openssl_shutdown_locks(void)
+{
+    int c, maxlocks = CRYPTO_num_locks();
+
+    gw_assert(ssl_static_locks != NULL);
+
+    /* remove call-back from the locks */
+    CRYPTO_set_locking_callback(NULL);
+
+    for (c = 0; c < maxlocks; c++) 
+        mutex_destroy(ssl_static_locks[c]);
+
+    gw_free(ssl_static_locks);
+    ssl_static_locks = NULL;
+}
+
+void conn_init_ssl(void)
+{
     SSL_library_init();
     SSL_load_error_strings();
-    global_ssl_context = SSL_CTX_new(SSLv23_method());
+    global_ssl_context = SSL_CTX_new(SSLv23_client_method());
+    SSL_CTX_set_mode(global_ssl_context, 
+        SSL_MODE_ENABLE_PARTIAL_WRITE | SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER);
 }
 
 void server_ssl_init(void) 
 {
-    int c, maxlocks = CRYPTO_num_locks();
-    
-    gw_assert(ssl_server_static_locks == NULL);
-    ssl_server_static_locks = gw_malloc(sizeof(Mutex *) * maxlocks);
-    for (c=0;c<maxlocks;c++) 
-         ssl_server_static_locks[c] = mutex_create();
-
-    CRYPTO_set_locking_callback(openssl_server_locking_function);
-    CRYPTO_set_id_callback((CRYPTO_CALLBACK_PTR)gwthread_self);
-
     SSLeay_add_ssl_algorithms();
     SSL_load_error_strings();
     global_server_ssl_context = SSL_CTX_new(SSLv23_server_method());
+    SSL_CTX_set_mode(global_server_ssl_context, 
+        SSL_MODE_ENABLE_PARTIAL_WRITE | SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER);
     if (!SSL_CTX_set_default_verify_paths(global_server_ssl_context)) {
 	   panic(0, "can not set default path for server");
     }
@@ -1256,25 +1292,20 @@ void server_ssl_init(void)
 
 void conn_shutdown_ssl(void)
 {
-    int c, maxlocks = CRYPTO_num_locks();
-
     if (global_ssl_context)
         SSL_CTX_free(global_ssl_context);
-
-    for (c=0;c<maxlocks;c++) 
-        mutex_destroy(ssl_static_locks[c]);
-    gw_free(ssl_static_locks);
+    
+    ERR_free_strings();
+    EVP_cleanup();
 }
 
 void server_shutdown_ssl(void)
 {
-    int c, maxlocks = CRYPTO_num_locks();
+    if (global_server_ssl_context)
+        SSL_CTX_free(global_server_ssl_context);
 
-    SSL_CTX_free(global_server_ssl_context);
-
-    for (c=0;c<maxlocks;c++) 
-        mutex_destroy(ssl_server_static_locks[c]);
-    gw_free(ssl_server_static_locks);
+    ERR_free_strings();
+    EVP_cleanup();
 }
 
 void use_global_client_certkey_file(Octstr *certkeyfile)
@@ -1286,10 +1317,10 @@ void use_global_client_certkey_file(Octstr *certkeyfile)
                                 octstr_get_cstr(certkeyfile),
                                 SSL_FILETYPE_PEM);
     if (SSL_CTX_check_private_key(global_ssl_context) != 1)
-        panic(0, "reading global client certificate file %s, the certificate "
+        panic(0, "reading global client certificate file `%s', the certificate "
 	      "isn't consistent with the private key (or failed reading the file)", 
               octstr_get_cstr(certkeyfile));
-    info(0, "Using global SSL certificate and key from file %s",
+    info(0, "Using global SSL certificate and key from file `%s'",
          octstr_get_cstr(certkeyfile));
 }
 
@@ -1308,8 +1339,8 @@ void use_global_server_certkey_file(Octstr *certfile, Octstr *keyfile)
                   (or failed reading the file)", 
                   octstr_get_cstr(certfile), octstr_get_cstr(keyfile));
     }
-    info(0, "Using global server SSL certificate from file %s", octstr_get_cstr(certfile));
-    info(0, "Using global server SSL key from file %s", octstr_get_cstr(keyfile));
+    info(0, "Using global server SSL certificate from file `%s'", octstr_get_cstr(certfile));
+    info(0, "Using global server SSL key from file `%s'", octstr_get_cstr(keyfile));
 }
 
 static int verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
@@ -1328,6 +1359,28 @@ static int verify_callback(int preverify_ok, X509_STORE_CTX *ctx)
     return preverify_ok;
 }
 
+void use_global_trusted_ca_file(Octstr *ssl_trusted_ca_file)
+{
+    if (ssl_trusted_ca_file != NULL) {
+	if (!SSL_CTX_load_verify_locations(global_ssl_context,
+					   octstr_get_cstr(ssl_trusted_ca_file),
+					   NULL)) {
+	    panic(0, "Failed to load SSL CA file: %s", octstr_get_cstr(ssl_trusted_ca_file));
+	} else {
+	    info(0, "Using CA root certificates from file %s",
+		 octstr_get_cstr(ssl_trusted_ca_file));
+	    SSL_CTX_set_verify(global_ssl_context,
+			       SSL_VERIFY_PEER,
+			       verify_callback);
+	}
+	
+    } else {
+	SSL_CTX_set_verify(global_ssl_context,
+			   SSL_VERIFY_NONE,
+			   NULL);
+    }
+}
+
 void conn_config_ssl (CfgGroup *grp)
 {
     Octstr *ssl_client_certkey_file = NULL;
@@ -1354,25 +1407,8 @@ void conn_config_ssl (CfgGroup *grp)
 
     ssl_trusted_ca_file = cfg_get(grp, octstr_imm("ssl-trusted-ca-file"));
     
-    if (ssl_trusted_ca_file != NULL) {
-	if (!SSL_CTX_load_verify_locations(global_ssl_context,
-					   octstr_get_cstr(ssl_trusted_ca_file),
-					   NULL)) {
-	    panic(0, "Failed to load SSL CA file: %s", octstr_get_cstr(ssl_trusted_ca_file));
-	} else {
-	    info(0, "Using CA root certificates from file %s",
-		 octstr_get_cstr(ssl_trusted_ca_file));
-	    SSL_CTX_set_verify(global_ssl_context,
-			       SSL_VERIFY_PEER,
-			       verify_callback);
-	}
-	
-    } else {
-	SSL_CTX_set_verify(global_ssl_context,
-			   SSL_VERIFY_NONE,
-			   NULL);
-    }
-    
+    use_global_trusted_ca_file(ssl_trusted_ca_file);
+
     octstr_destroy(ssl_client_certkey_file);
     octstr_destroy(ssl_server_cert_file);
     octstr_destroy(ssl_server_key_file);
