@@ -1,3 +1,59 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2005 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*
  * A very simple push initiator for testing a push proxy gateway
  *
@@ -25,17 +81,30 @@ static long max_pushes = 1;
 static int verbose = 1,
            use_hardcoded = 0,
            num_urls = 0,
-           wait = 0,
            use_headers = 0,
-           use_config = 0;
+           use_config = 0,
+           accept_binary = 0, 
+           use_numeric = 0,
+           use_string = 0,
+           use_content_header = 0,
+           add_epilogue = 0,
+           add_preamble = 0,
+           use_dlr_mask = 0,
+           use_dlr_url = 0;
 static double wait_seconds = 0.0;
 static Counter *counter = NULL;
 static char **push_data = NULL;
 static char *boundary = NULL;
 static Octstr *content_flag = NULL;
 static Octstr *appid_flag = NULL;
+static Octstr *appid_string = NULL;
+static Octstr *content_header = NULL;
 static Octstr *content_transfer_encoding = NULL;
 static Octstr *connection = NULL;
+static Octstr *delimiter = NULL;
+static Octstr *initiator_uri = NULL;
+static Octstr *dlr_mask = NULL;
+static Octstr *dlr_url = NULL;
 
 enum { SSL_CONNECTION_OFF = 0,
        DEFAULT_NUMBER_OF_RELOGS = 2};
@@ -90,73 +159,128 @@ static void read_test_ppg_config(Octstr *name)
     cfg_destroy(cfg);
 }
 
-static void add_push_application_id(List **push_headers, Octstr *appid_flag)
+static void add_delimiter(Octstr **content)
 {
-    if (octstr_compare(appid_flag, octstr_imm("any")) == 0)
-        http_header_add(*push_headers, "X-WAP-Application-Id", 
-                        "http://www.wiral.com:*");
-    else if (octstr_compare(appid_flag, octstr_imm("ua")) == 0)
-        http_header_add(*push_headers, "X-WAP-Application-Id", 
-                        "http://www.wiral.com:wml.ua");
-    else if (octstr_compare(appid_flag, octstr_imm("mms")) == 0)
-        http_header_add(*push_headers, "X-WAP-Application-Id", 
-                        "mms.ua");
-    else if (octstr_compare(appid_flag, octstr_imm("scrap")) == 0)
-        http_header_add(*push_headers, "X-WAP-Application-Id", 
-                        "no appid at all");
+    if (octstr_compare(delimiter, octstr_imm("crlf")) == 0) {
+        octstr_format_append(*content, "%c", '\r');
+    }
+
+    octstr_format_append(*content, "%c", '\n');
 }
 
+static void add_push_application_id(List **push_headers, Octstr *appid_flag,
+                                    int use_string)
+{
+    if (use_string) {
+        gwlist_append(*push_headers, appid_string);
+        return;
+    }
+
+    if (octstr_compare(appid_flag, octstr_imm("any")) == 0) {
+        if (!use_numeric)
+            http_header_add(*push_headers, "X-WAP-Application-Id", 
+                            "http://www.wiral.com:*");
+        else
+            http_header_add(*push_headers, "X-WAP-Application-Id", "0");
+    } else if (octstr_compare(appid_flag, octstr_imm("ua")) == 0) {
+        if (!use_numeric)
+            http_header_add(*push_headers, "X-WAP-Application-Id", 
+                            "http://www.wiral.com:wml.ua");
+        else
+            http_header_add(*push_headers, "X-WAP-Application-Id", "2");
+    } else if (octstr_compare(appid_flag, octstr_imm("mms")) == 0) {
+        if (!use_numeric)
+            http_header_add(*push_headers, "X-WAP-Application-Id", 
+                            "mms.ua");
+        else
+            http_header_add(*push_headers, "X-WAP-Application-Id", "4");
+    } else if (octstr_compare(appid_flag, octstr_imm("scrap")) == 0) {
+        if (!use_numeric)
+            http_header_add(*push_headers, "X-WAP-Application-Id", 
+                        "no appid at all");
+        else
+            http_header_add(*push_headers, "X-WAP-Application-Id", 
+                            "this is not a numeric header");
+    }
+}
+
+static void add_dlr_mask(List **push_headers, Octstr *value)
+{
+    http_header_add(*push_headers, "X-Kannel-DLR-Mask", 
+                    octstr_get_cstr(value));
+}
+
+static void add_dlr_url(List **push_headers, Octstr *value)
+{
+    http_header_add(*push_headers, "X-Kannel-DLR-Url",
+                    octstr_get_cstr(value));
+}
+
+static void add_part_header(Octstr *content_keader, Octstr **wap_content)
+{
+    if (use_content_header) {
+        octstr_append(*wap_content, content_header);
+    }
+
+    add_delimiter(wap_content);
+}
+
+
 static void add_content_type(Octstr *content_flag, Octstr **wap_content)
 {
     if (octstr_compare(content_flag, octstr_imm("wml")) == 0)
         *wap_content = octstr_format("%s", 
-            "Content-Type: text/vnd.wap.wml\r\n");
+            "Content-Type: text/vnd.wap.wml");
     else if (octstr_compare(content_flag, octstr_imm("si")) == 0)
 	    *wap_content = octstr_format("%s",
-            "Content-Type: text/vnd.wap.si\r\n");
+            "Content-Type: text/vnd.wap.si");
     else if (octstr_compare(content_flag, octstr_imm("sl")) == 0)
 	    *wap_content = octstr_format("%s",
-            "Content-Type: text/vnd.wap.sl\r\n");
+            "Content-Type: text/vnd.wap.sl");
     else if (octstr_compare(content_flag, octstr_imm("multipart")) == 0)
         *wap_content = octstr_format("%s",
-            "Content-Type: multipart/related; boundary=fsahgwruijkfldsa\r\n");
+            "Content-Type: multipart/related; boundary=fsahgwruijkfldsa");
     else if (octstr_compare(content_flag, octstr_imm("mms")) == 0) 
         *wap_content = octstr_format("%s", 
-            "Content-Type: application/vnd.wap.mms-message\r\n"); 
+            "Content-Type: application/vnd.wap.mms-message"); 
     else if (octstr_compare(content_flag, octstr_imm("scrap")) == 0)
-        *wap_content = octstr_format("%s", "no type at all\r\n"); 
+        *wap_content = octstr_format("%s", "no type at all"); 
     else if (octstr_compare(content_flag, octstr_imm("nil")) == 0)
         *wap_content = octstr_create("");
+    if (octstr_len(*wap_content) > 0)
+        add_delimiter(wap_content);
 }
 
 static void add_content_transfer_encoding_type(Octstr *content_flag, 
                                                Octstr *wap_content)
 {
     if (!content_flag)
-	     return;
+	return;
 
     if (octstr_compare(content_flag, octstr_imm("base64")) == 0)
-	    octstr_append_cstr(wap_content, "Content-transfer-encoding: base64\r\n");
+	octstr_append_cstr(wap_content, "Content-transfer-encoding: base64");
+
+    add_delimiter(&wap_content);
 }
 
-static void add_connection_header(Octstr *connection, Octstr *wap_content)
+static void add_connection_header(List **push_headers, Octstr *connection)
 {
     if (!connection)
         return;
 
     if (octstr_compare(connection, octstr_imm("close")) == 0)
-        octstr_format_append(wap_content, "%s", "Connection: close\r\n");
+        http_header_add(*push_headers, "Connection", "close");
     else if (octstr_compare(connection, octstr_imm("keep-alive")) == 0) 
-        octstr_format_append(wap_content, "%s", "Connection: keep-alive\r\n");
+        http_header_add(*push_headers, "Connection", "keep-alive");
 }
 
 static void transfer_encode (Octstr *cte, Octstr *content)
 {
     if (!cte)
-	    return;
+	return;
     
     if (octstr_compare(cte, octstr_imm("base64")) == 0) {
-	    octstr_binary_to_base64(content);
+       octstr_binary_to_base64(content);
     }
 }
 
@@ -180,12 +304,10 @@ static Octstr *make_part_delimiter(Octstr *boundary)
     Octstr *part_delimiter;
 
     part_delimiter = octstr_create("");
-    octstr_format_append(part_delimiter, "%c", '\r');
-    octstr_format_append(part_delimiter, "%c", '\n');
+    add_delimiter(&part_delimiter);
     octstr_format_append(part_delimiter, "%s", "--");
     octstr_append(part_delimiter, boundary);
-    octstr_format_append(part_delimiter, "%c", '\r');
-    octstr_format_append(part_delimiter, "%c", '\n');
+    add_delimiter(&part_delimiter);
     
     return part_delimiter;
 }
@@ -195,14 +317,11 @@ static Octstr *make_close_delimiter(Octstr *boundary)
     Octstr *close_delimiter;
 
     close_delimiter = octstr_create("");
-    octstr_format_append(close_delimiter, "%c", '\r');
-    octstr_format_append(close_delimiter, "%c", '\n');
+    add_delimiter(&close_delimiter);
     octstr_format_append(close_delimiter, "%s", "--");
     octstr_append(close_delimiter, boundary);
     octstr_format_append(close_delimiter, "%s", "--");
-    octstr_format_append(close_delimiter, "%c", '\r');
-    octstr_format_append(close_delimiter, "%c", '\n');
-    
+    /*add_delimiter(&close_delimiter);*/
 
     return close_delimiter;
 }
@@ -222,10 +341,20 @@ static List *push_headers_create(size_t content_len)
                         octstr_get_cstr(mos = make_multipart_value(boundary)));
     if (use_headers)
         http_add_basic_auth(push_headers, username, password);
-    add_push_application_id(&push_headers, appid_flag);
+    add_push_application_id(&push_headers, appid_flag, use_string);
+    add_connection_header(&push_headers, connection);
+    if (use_dlr_mask)
+        add_dlr_mask(&push_headers, dlr_mask);
+    if (use_dlr_url)
+        add_dlr_url(&push_headers, dlr_url);
 
     octstr_destroy(mos);
 
+    /* add initiator... */
+    if (initiator_uri)
+	http_header_add(push_headers, "X-Wap-Initiator-URI",
+			octstr_get_cstr(initiator_uri));
+
     return push_headers;
 }
 
@@ -286,20 +415,33 @@ static Octstr *push_content_create(void)
         add_content_type(content_flag, &wap_content);
         add_content_transfer_encoding_type(content_transfer_encoding, 
                                            wap_content);
-        add_connection_header(connection, wap_content);
+        add_part_header(content_header, &wap_content);
 
+        /* Read the content file. (To be pushed)*/
         if ((wap_file_content = 
                 octstr_read_file(octstr_get_cstr(content_file))) == NULL)
 	         panic(0, "Stopping");
+        if (accept_binary) {
+            octstr_delete_matching(wap_file_content, octstr_imm(" "));
+            octstr_delete_matching(wap_file_content, octstr_imm("\n"));
+            octstr_delete_matching(wap_file_content, octstr_imm("\r"));
+            if (!octstr_is_all_hex(wap_file_content))
+                panic(0, "non-hex chars in the content file, cannot continue");
+            octstr_hex_to_binary(wap_file_content);            
+        }
 
-	    transfer_encode (content_transfer_encoding, wap_file_content);
+	transfer_encode(content_transfer_encoding, wap_file_content);
         octstr_append(wap_content, wap_file_content);
         octstr_destroy(wap_file_content);
 
-        pap_content = octstr_format("%s", "Content-Type: application/xml\r\n");
+        /* Read the control file. (To control pushing)*/
+        pap_content = octstr_format("%s", "Content-Type: application/xml");
+        add_delimiter(&pap_content);
+        add_delimiter(&pap_content);
         if ((pap_file_content = 
                 octstr_read_file(octstr_get_cstr(pap_file))) ==  NULL)
 	        panic(0, "Stopping");
+        
         octstr_append(pap_content, pap_file_content);
         octstr_destroy(pap_file_content);
 
@@ -307,14 +449,22 @@ static Octstr *push_content_create(void)
 	        panic(0, "Cannot open the push content files");
 
         push_content = octstr_create("");
+        if (add_preamble)
+            octstr_append(push_content, octstr_imm("the parser should discard this"));
         octstr_append(push_content, 
             bpos = make_part_delimiter(octstr_imm(boundary)));
+        /*octstr_append(push_content, octstr_imm("\r\n"));*/ /* Do we accept an additional 
+                                                          * clrf ? */
         octstr_append(push_content, pap_content);
         octstr_append(push_content, bpos);
         octstr_destroy(bpos);
         octstr_append(push_content, wap_content);
         octstr_append(push_content, 
             bcos = make_close_delimiter(octstr_imm(boundary)));
+        if (add_epilogue) {
+            octstr_append(push_content, octstr_imm("\r\n"));
+            octstr_append(push_content, octstr_imm("the parser should discard this"));
+        }
         octstr_destroy(bcos);
         octstr_destroy(pap_content);
         octstr_destroy(wap_content);
@@ -449,7 +599,7 @@ static int receive_push_reply(HTTPCaller *caller)
     if (verbose)
         debug("test.ppg", 0, "TEST_PPG: reply headers were");
 
-    while ((os = list_extract_first(reply_headers)) != NULL) {
+    while ((os = gwlist_extract_first(reply_headers)) != NULL) {
         if (verbose)
             octstr_dump(os, 0); 
         octstr_destroy(os);
@@ -518,7 +668,7 @@ static void push_thread(void *arg)
             if (i >= max_pushes)
 	            goto receive_rest;
             start_push(caller, i);
-            if (wait)
+            if (wait_seconds > 0)
                 gwthread_sleep(wait_seconds);
             ++in_queue;
         }
@@ -574,6 +724,20 @@ static void help(void)
     info(0, "-a application id");
     info(0, "Define the client application that will handle the push. Any,"); 
     info(0, "ua, mms, nil and scrap accepted, default ua.");
+    info(0, "-n");
+    info(0, "if set, use numeric appid values instead of string ones. For");
+    info(0, "instance, '4' instead of 'mms.ua'. Default is off.");
+    info(0, "-s string");
+    info(0, "supply a message header as a plain string. For instance"); 
+    info(0, "-s x-wap-application-id:mms.ua equals -a ua. Default is");
+    info(0, "x-wap-application-id:mms.ua.");
+    info(0, "-I string");
+    info(0, "supply an initiator header as a plain string. For instance"); 
+    info(0, "-I x-wap-application-id:http://foo.bar equals -I http://foo.bar");
+    info(0, "-S string");
+    info(0, "supply an additional part header (for push content) as a string."); 
+    info(0, "For instance, -S Content-Language: en. Default no additional part");
+    info(0, "headers.");
     info(0, "-b");
     info(0, "If true, send username/password in headers. Default false");
     info(0, "-v number");
@@ -598,6 +762,23 @@ static void help(void)
     info(0, "Default: read components from files");
     info(0, "-t");
     info(0, "number of threads, maximum 1024, default 1");
+    info(0, "-B");
+    info(0, "accept binary push content. Default: off.");
+    info(0, "Binary content consist of hex numbers. In addition, crs, lfs and");
+    info(0, "spaces are accepted, and ignored.");
+    info(0, "-d value");
+    info(0, "set delimiter to be used. Accepted values crlf and lf. Default crlf.");
+    info(0, "-E");
+    info(0, "If set, add a hardcoded epilogue (epilogue is to be discarded anyway).");
+    info(0, "Default off.");
+    info(0, "-p");
+    info(0, "If set, add hardcoded preamble. Default is off.");
+    info(0, "-m value");
+    info(0, "If set, add push header X-Kannel-DLR-Mask: value");
+    info(0, "Default off.");
+    info(0, "-u value");
+    info(0, "If set, add push header X-Kannel-DLR-Url: value");
+    info(0, "Default off.");
 }
 
 int main(int argc, char **argv)
@@ -614,7 +795,7 @@ int main(int argc, char **argv)
     gwlib_init();
     num_threads = 1;
 
-    while ((opt = getopt(argc, argv, "Hhbv:qr:t:c:a:i:e:k:")) != EOF) {
+    while ((opt = getopt(argc, argv, "HhBbnEpv:qr:t:c:a:i:e:k:d:s:S:I:m:u:")) != EOF) {
         switch(opt) {
 	    case 'v':
 	        log_set_output_level(atoi(optarg));
@@ -629,14 +810,13 @@ int main(int argc, char **argv)
 	    break; 
             
 	    case 'i': 
-	        wait = 1;
-            wait_seconds = atof(optarg);
+                wait_seconds = atof(optarg);
 	    break;
 
-        case 't': 
+            case 't': 
 	        num_threads = atoi(optarg);
-            if (num_threads > MAX_THREADS)
-		        num_threads = MAX_THREADS;
+                if (num_threads > MAX_THREADS)
+		    num_threads = MAX_THREADS;
 	    break;
 
 	    case 'H': 
@@ -651,52 +831,62 @@ int main(int argc, char **argv)
                     octstr_compare(content_flag, octstr_imm("nil")) != 0 &&
                     octstr_compare(content_flag, octstr_imm("mms")) != 0 &&
                     octstr_compare(content_flag, octstr_imm("scrap")) != 0 &&
-                    octstr_compare(content_flag, 
-                    octstr_imm("multipart")) != 0){
+                    octstr_compare(content_flag, octstr_imm("multipart")) != 0) {
 		        octstr_destroy(content_flag);
 		        error(0, "TEST_PPG: Content type not known");
 		        help();
-                exit(1);
+                         exit(1);
             }
 	    break;
 
 	    case 'a':
 	        appid_flag = octstr_create(optarg);
-            if (octstr_compare(appid_flag, octstr_imm("any")) != 0 && 
-                    octstr_compare(appid_flag, octstr_imm("ua")) != 0 &&
-                    octstr_compare(appid_flag, octstr_imm("mms")) != 0 &&
-                    octstr_compare(appid_flag, octstr_imm("nil")) != 0 &&
-                    octstr_compare(appid_flag, octstr_imm("scrap")) != 0) {
-		        octstr_destroy(appid_flag);
-		        error(0, "TEST_PPG: Push application id not known");
-		        help();
+                if (octstr_compare(appid_flag, octstr_imm("any")) != 0 && 
+                        octstr_compare(appid_flag, octstr_imm("ua")) != 0 &&
+                        octstr_compare(appid_flag, octstr_imm("mms")) != 0 &&
+                        octstr_compare(appid_flag, octstr_imm("nil")) != 0 &&
+                        octstr_compare(appid_flag, octstr_imm("scrap")) != 0) {
+		octstr_destroy(appid_flag);
+		error(0, "TEST_PPG: Push application id not known");
+		help();
                 exit(1);
            }
 	    break;
 
+            case 'n':
+                use_numeric = 1;
+            break;
+
+            case 's':
+                appid_string = octstr_create(optarg);
+                use_string = 1;
+            break;
+
+            case 'S':
+                content_header = octstr_create(optarg);
+                use_content_header = 1;
+            break;
+
 	    case 'e':
-		    content_transfer_encoding = octstr_create(optarg);
-            if (octstr_compare(content_transfer_encoding, 
-                               octstr_imm("base64")) != 0) {
-		        octstr_destroy(content_transfer_encoding);
-		        error(0, "TEST_PPG: unknown content transfer" 
-                      " encoding \"%s\"",
-			          octstr_get_cstr(content_transfer_encoding));
-		        help();
-                exit(1);
-		    }
+		content_transfer_encoding = octstr_create(optarg);
+                if (octstr_compare(content_transfer_encoding, octstr_imm("base64")) != 0) {
+	            octstr_destroy(content_transfer_encoding);
+		    error(0, "TEST_PPG: unknown content transfer" 
+                      " encoding \"%s\"", octstr_get_cstr(content_transfer_encoding));
+		    help();
+                    exit(1);
+		}
 	    break;
 
 	    case 'k':
 	        connection = octstr_create(optarg);
-            if (octstr_compare(connection, octstr_imm("close")) != 0 && 
-                    octstr_compare(connection, 
-                        octstr_imm("keep-alive")) != 0) {
-		        octstr_destroy(connection);
-		        error(0, "TEST_PPG: Connection-header unacceptable");
-		        help();
-                exit(1);
-           }
+                if (octstr_compare(connection, octstr_imm("close")) != 0 && 
+                        octstr_compare(connection, octstr_imm("keep-alive")) != 0) {
+	            octstr_destroy(connection);
+		    error(0, "TEST_PPG: Connection-header unacceptable");
+		    help();
+                    exit(1);
+                }
 	    break;
 
 	    case 'h':
@@ -707,7 +897,44 @@ int main(int argc, char **argv)
 	        use_headers = 1;
 	    break;
 
-	    case '?':
+            case 'B':
+                accept_binary = 1;
+            break;
+
+            case 'd':
+                delimiter = octstr_create(optarg);
+                if (octstr_compare(delimiter, octstr_imm("crlf")) != 0 &&
+                        octstr_compare(delimiter, octstr_imm("lf")) != 0) {
+                    octstr_destroy(delimiter);
+                    error(0, "illegal d value");
+                    help();
+                    exit(1);
+                }
+            break;
+
+            case 'E':
+                add_epilogue = 1;
+            break;
+
+            case 'p':
+                add_preamble = 1;
+            break;
+
+            case 'I':
+                initiator_uri = octstr_create(optarg);
+		break;
+
+            case 'm':
+                use_dlr_mask = 1;
+                dlr_mask = octstr_create(optarg);
+            break;
+
+            case 'u':
+                use_dlr_url = 1;
+                dlr_url = octstr_create(optarg);
+            break;
+
+	case '?':
 	    default:
 	        error(0, "TEST_PPG: Invalid option %c", opt);
             help();
@@ -730,6 +957,21 @@ int main(int argc, char **argv)
     if (appid_flag == NULL)
         appid_flag = octstr_imm("ua");
 
+    if (appid_string == NULL)
+        appid_string = octstr_imm("x-wap-application-id: wml.ua");
+
+    if (content_header == NULL)
+        use_content_header = 0;
+
+    if (dlr_mask == NULL)
+        use_dlr_mask = 0;
+
+    if (dlr_url == NULL)
+        use_dlr_url = 0;
+
+    if (delimiter == NULL)
+        delimiter = octstr_imm("crlf");
+
     if (use_hardcoded) {
         username = octstr_imm("troo");
         password = octstr_imm("far");
@@ -785,6 +1027,7 @@ int main(int argc, char **argv)
 
     octstr_destroy(content_flag);
     octstr_destroy(appid_flag);
+    octstr_destroy(content_header);
     octstr_destroy(content_file);
     octstr_destroy(pap_file);
     octstr_destroy(ssl_client_certkey_file);
@@ -792,6 +1035,9 @@ int main(int argc, char **argv)
     octstr_destroy(password);
     octstr_destroy(push_url);
     octstr_destroy(connection);
+    octstr_destroy(delimiter);
+    octstr_destroy(dlr_mask);
+    octstr_destroy(dlr_url);
     counter_destroy(counter);
     gwlib_shutdown();
 
