@@ -1,3 +1,59 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2005 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*
  * wtls.c: WTLS server-side implementation
  *
@@ -138,11 +194,11 @@ WAPEvent *wtls_unpack_wdp_datagram(Msg *msg)
 
 void wtls_init(void) {
         /* Initialise our various lists and counters */
-        wtls_machines = list_create();
+        wtls_machines = gwlist_create();
         wtls_machine_id_counter = counter_create();
         
-        wtls_queue = list_create();
-        list_add_producer(wtls_queue);
+        wtls_queue = gwlist_create();
+        gwlist_add_producer(wtls_queue);
 
         /* Idiot check - ensure that we are able to start running */
         gw_assert(wtls_run_status == limbo);
@@ -155,22 +211,22 @@ void wtls_shutdown(void) {
            prepare for termination */
         gw_assert(wtls_run_status == running);
         wtls_run_status = terminating;
-        list_remove_producer(wtls_queue);
+        gwlist_remove_producer(wtls_queue);
         gwthread_join_every(main_thread);
 
         /* Print out a friendly message stating that we're going to die */
         debug("wap.wtls", 0, "wtls_shutdown: %ld wtls machines left",
-              list_len(wtls_machines));
+              gwlist_len(wtls_machines));
 
         /* And clean up nicely after ourselves */
-        list_destroy(wtls_machines, wtls_machine_destroy);
-        list_destroy(wtls_queue, wap_event_destroy_item);     
+        gwlist_destroy(wtls_machines, wtls_machine_destroy);
+        gwlist_destroy(wtls_queue, wap_event_destroy_item);     
         counter_destroy(wtls_machine_id_counter);
 }
 
 void wtls_dispatch_event(WAPEvent *event) {
         /* Stick the event on the incoming events queue */
-        list_produce(wtls_queue, event);
+        gwlist_produce(wtls_queue, event);
 }
 
 int wtls_get_address_tuple(long mid, WAPAddrTuple **tuple) {
@@ -295,7 +351,7 @@ static void main_thread(void *arg) {
 	WAPEvent *e;
         
 	while (wtls_run_status == running && 
-               (e = list_consume(wtls_queue)) != NULL) {
+               (e = gwlist_consume(wtls_queue)) != NULL) {
 		sm = wtls_machine_find_or_create(e);
 		if (sm == NULL)
 			wap_event_destroy(e);
@@ -472,7 +528,7 @@ static WTLSMachine *wtls_machine_find(WAPAddrTuple *tuple,
 	pat.tuple = tuple;
 	pat.mid = mid;
 	
-	m = list_search(wtls_machines, &pat, is_wanted_wtls_machine);
+	m = gwlist_search(wtls_machines, &pat, is_wanted_wtls_machine);
 	return m;
 }
 
@@ -490,7 +546,7 @@ static WTLSMachine *wtls_machine_create(WAPAddrTuple *tuple) {
         #define PDULIST(name) wtls_machine->name = NULL;
         #include "wtls_machine-decl.h"
         
-        list_append(wtls_machines, wtls_machine);
+        gwlist_append(wtls_machines, wtls_machine);
         wtls_machine->mid = counter_increase(wtls_machine_id_counter);
         wtls_machine->addr_tuple = wap_addr_tuple_duplicate(tuple);
 
@@ -511,7 +567,7 @@ static void wtls_machine_destroy(void * p) {
        wtls_machine = p;
        debug("wap.wtls", 0, "WTLS: Destroying WTLSMachine %p (%ld)",
              (void *) wtls_machine, wtls_machine->mid);
-       list_delete_equal(wtls_machines, wtls_machine);        
+       gwlist_delete_equal(wtls_machines, wtls_machine);        
         
        #define MACHINE(field) field
        #define ENUM(name) wtls_machine->name = NULL_STATE;
@@ -581,7 +637,7 @@ static int wtls_machine_has_mid(void *a, void *b) {
 }
 
 static WTLSMachine *find_wtls_machine_using_mid(long mid) {
-       return list_search(wtls_machines, &mid, wtls_machine_has_mid);
+       return gwlist_search(wtls_machines, &mid, wtls_machine_has_mid);
 }
 
 /* Used for list searches */
