@@ -1,3 +1,59 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2005 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /* 
  * wtls_statesupport.c
  * 
@@ -429,9 +485,9 @@ void wtls_decrypt_pdu_list(WTLSMachine *wtls_machine, List *pdu_list)
     Octstr* decryptedData = NULL;
 	wtls_Payload *payload;
 	
-	listlen = list_len(pdu_list);
+	listlen = gwlist_len(pdu_list);
 	for( i=0; i<listlen; i++) {
-		payload = (wtls_Payload *)list_get(pdu_list, i);
+		payload = (wtls_Payload *)gwlist_get(pdu_list, i);
 		
 		if(payload->cipher) {
 			debug("wtls", 0, "Decrypting PDU %d", i);
@@ -560,14 +616,14 @@ CipherSuite* wtls_choose_ciphersuite(List* ciphersuites) {
 		int i = 0;
         int listLen;
 		
-		listLen = list_len(ciphersuites);
+		listLen = gwlist_len(ciphersuites);
 		
         //returnSuite = gw_malloc(sizeof(CipherSuite));
 
 		/* the first CS in the list */
 		do {
 			/* the next CS in the list */
-			currentCS = list_get(ciphersuites, i);
+			currentCS = gwlist_get(ciphersuites, i);
 			/* Check if we support this BulkCipher */
 			if(currentCS->bulk_cipher_algo >= RC5_CBC_40 &&
 			   currentCS->bulk_cipher_algo <= IDEA_CBC) {
@@ -606,11 +662,11 @@ int wtls_choose_clientkeyid(List* clientKeyIds) {
 		int i = 0;
 		int listLen;
 		
-		listLen = list_len(clientKeyIds);
+		listLen = gwlist_len(clientKeyIds);
 		debug("wtls", 0, "listLen = %d", listLen);
 		
 		do {
-			currentKeyId = list_get(clientKeyIds, i);
+			currentKeyId = gwlist_get(clientKeyIds, i);
 			debug("wtls", 0, "Key %d", i);
 			dump_key_exchange_id("wtls", 0, currentKeyId);
 			
@@ -719,10 +775,10 @@ void add_all_handshake_data(WTLSMachine *wtls_machine, List *pdu_list)
 
 	gw_assert(pdu_list != NULL);
 	
-	listlen = list_len(pdu_list);
+	listlen = gwlist_len(pdu_list);
 	debug("wtls", 0,"adding handshake data from %d PDU(s)", listlen);
 	for(i=0; i<listlen; i++) {
-		payload = (wtls_Payload *)list_get(pdu_list, i);
+		payload = (wtls_Payload *)gwlist_get(pdu_list, i);
 		if(payload->type == Handshake_PDU) {
             octstr_insert(wtls_machine->handshake_data, payload->data,
                           octstr_len(wtls_machine->handshake_data));
